# Stubs for six.moves
#
# Note: Commented out items means they weren't implemented at the time.
# Uncomment them when the modules have been added to the typeshed.
import sys

from io import StringIO as cStringIO
from builtins import filter as filter
from itertools import filterfalse as filterfalse
from builtins import input as input
from sys import intern as intern
from builtins import map as map
from os import getcwd as getcwd
from os import getcwdb as getcwdb
from builtins import range as range
from functools import reduce as reduce
from shlex import quote as shlex_quote
from io import StringIO as StringIO
from collections import UserDict as UserDict
from collections import UserList as UserList
from collections import UserString as UserString
from builtins import range as xrange
from builtins import zip as zip
from itertools import zip_longest as zip_longest
import builtins as builtins
import configparser as configparser
# import copyreg as copyreg
# import dbm.gnu as dbm_gnu
import _dummy_thread as _dummy_thread
import http.cookiejar as http_cookiejar
import http.cookies as http_cookies
import html.entities as html_entities
import html.parser as html_parser
import http.client as http_client
import email.mime.multipart as email_mime_multipart
import email.mime.nonmultipart as email_mime_nonmultipart
import email.mime.text as email_mime_text
import email.mime.base as email_mime_base
import http.server as BaseHTTPServer
import http.server as CGIHTTPServer
import http.server as SimpleHTTPServer
import pickle as cPickle
import queue as queue
import reprlib as reprlib
import socketserver as socketserver
import _thread as _thread
import tkinter as tkinter
# import tkinter.dialog as tkinter_dialog
# import tkinter.filedialog as tkinter_filedialog
# import tkinter.scrolledtext as tkinter_scrolledtext
# import tkinter.simpledialog as tkinter_simpledialog
# import tkinter.tix as tkinter_tix
import tkinter.ttk as tkinter_ttk
import tkinter.constants as tkinter_constants
# import tkinter.dnd as tkinter_dnd
# import tkinter.colorchooser as tkinter_colorchooser
# import tkinter.commondialog as tkinter_commondialog
# import tkinter.filedialog as tkinter_tkfiledialog
# import tkinter.font as tkinter_font
# import tkinter.messagebox as tkinter_messagebox
# import tkinter.simpledialog as tkinter_tksimpledialog
import urllib.parse as urllib_parse
import urllib.error as urllib_error
import six.moves.urllib as urllib
import urllib.robotparser as urllib_robotparser
# import xmlrpc.client as xmlrpc_client
# import xmlrpc.server as xmlrpc_server

if sys.version_info >= (3, 4):
    from importlib import reload as reload_module
else:
    from imp import reload as reload_module



# Stubs for hmac

from typing import Any, Callable, Optional, Union, overload
from types import ModuleType
import sys

_B = Union[bytes, bytearray]

# TODO more precise type for object of hashlib
_Hash = Any

digest_size: None

if sys.version_info >= (3, 4):
    def new(key: _B, msg: Optional[_B] = ...,
            digestmod: Optional[Union[str, Callable[[], _Hash], ModuleType]] = ...) -> HMAC: ...
else:
    def new(key: _B, msg: Optional[_B] = ...,
            digestmod: Optional[Union[Callable[[], _Hash], ModuleType]] = ...) -> HMAC: ...

class HMAC:
    if sys.version_info >= (3,):
        digest_size = ...  # type: int
    if sys.version_info >= (3, 4):
        block_size = ...  # type: int
        name = ...  # type: str
    def update(self, msg: _B) -> None: ...
    def digest(self) -> bytes: ...
    def hexdigest(self) -> str: ...
    def copy(self) -> HMAC: ...

@overload
def compare_digest(a: str, b: str) -> bool: ...
@overload
def compare_digest(a: bytes, b: bytes) -> bool: ...
@overload
def compare_digest(a: bytearray, b: bytearray) -> bool: ...

if sys.version_info >= (3, 7):
    def digest(key: _B, msg: _B, digest: str) -> bytes: ...



from typing import Any

class _Missing:
    def __reduce__(self): ...

class _DictAccessorProperty:
    read_only = ...  # type: Any
    name = ...  # type: Any
    default = ...  # type: Any
    load_func = ...  # type: Any
    dump_func = ...  # type: Any
    __doc__ = ...  # type: Any
    def __init__(self, name, default=None, load_func=None, dump_func=None, read_only=None, doc=None): ...
    def __get__(self, obj, type=None): ...
    def __set__(self, obj, value): ...
    def __delete__(self, obj): ...

def _easteregg(app=None): ...



from typing import Any

from .message import Message

class Error(Exception): ...
class TypeTransformationError(Error): ...

class DescriptorMetaclass(type):
    def __instancecheck__(cls, obj): ...

class DescriptorBase:
    __metaclass__ = DescriptorMetaclass
    has_options = ...  # type: Any
    def __init__(self, options, options_class_name) -> None: ...
    def GetOptions(self): ...

class _NestedDescriptorBase(DescriptorBase):
    name = ...  # type: Any
    full_name = ...  # type: Any
    file = ...  # type: Any
    containing_type = ...  # type: Any
    def __init__(self, options, options_class_name, name, full_name, file, containing_type, serialized_start=..., serialized_end=...) -> None: ...
    def GetTopLevelContainingType(self): ...
    def CopyToProto(self, proto): ...

class Descriptor(_NestedDescriptorBase):
    def __new__(cls, name, full_name, filename, containing_type, fields, nested_types, enum_types, extensions, options=..., is_extendable=..., extension_ranges=..., oneofs=..., file=..., serialized_start=..., serialized_end=..., syntax=...): ...
    fields = ...  # type: Any
    fields_by_number = ...  # type: Any
    fields_by_name = ...  # type: Any
    nested_types = ...  # type: Any
    nested_types_by_name = ...  # type: Any
    enum_types = ...  # type: Any
    enum_types_by_name = ...  # type: Any
    enum_values_by_name = ...  # type: Any
    extensions = ...  # type: Any
    extensions_by_name = ...  # type: Any
    is_extendable = ...  # type: Any
    extension_ranges = ...  # type: Any
    oneofs = ...  # type: Any
    oneofs_by_name = ...  # type: Any
    syntax = ...  # type: Any
    def __init__(self, name, full_name, filename, containing_type, fields, nested_types, enum_types, extensions, options=..., is_extendable=..., extension_ranges=..., oneofs=..., file=..., serialized_start=..., serialized_end=..., syntax=...) -> None: ...
    def EnumValueName(self, enum, value): ...
    def CopyToProto(self, proto): ...

class FieldDescriptor(DescriptorBase):
    TYPE_DOUBLE = ...  # type: Any
    TYPE_FLOAT = ...  # type: Any
    TYPE_INT64 = ...  # type: Any
    TYPE_UINT64 = ...  # type: Any
    TYPE_INT32 = ...  # type: Any
    TYPE_FIXED64 = ...  # type: Any
    TYPE_FIXED32 = ...  # type: Any
    TYPE_BOOL = ...  # type: Any
    TYPE_STRING = ...  # type: Any
    TYPE_GROUP = ...  # type: Any
    TYPE_MESSAGE = ...  # type: Any
    TYPE_BYTES = ...  # type: Any
    TYPE_UINT32 = ...  # type: Any
    TYPE_ENUM = ...  # type: Any
    TYPE_SFIXED32 = ...  # type: Any
    TYPE_SFIXED64 = ...  # type: Any
    TYPE_SINT32 = ...  # type: Any
    TYPE_SINT64 = ...  # type: Any
    MAX_TYPE = ...  # type: Any
    CPPTYPE_INT32 = ...  # type: Any
    CPPTYPE_INT64 = ...  # type: Any
    CPPTYPE_UINT32 = ...  # type: Any
    CPPTYPE_UINT64 = ...  # type: Any
    CPPTYPE_DOUBLE = ...  # type: Any
    CPPTYPE_FLOAT = ...  # type: Any
    CPPTYPE_BOOL = ...  # type: Any
    CPPTYPE_ENUM = ...  # type: Any
    CPPTYPE_STRING = ...  # type: Any
    CPPTYPE_MESSAGE = ...  # type: Any
    MAX_CPPTYPE = ...  # type: Any
    LABEL_OPTIONAL = ...  # type: Any
    LABEL_REQUIRED = ...  # type: Any
    LABEL_REPEATED = ...  # type: Any
    MAX_LABEL = ...  # type: Any
    MAX_FIELD_NUMBER = ...  # type: Any
    FIRST_RESERVED_FIELD_NUMBER = ...  # type: Any
    LAST_RESERVED_FIELD_NUMBER = ...  # type: Any
    def __new__(cls, name, full_name, index, number, type, cpp_type, label, default_value, message_type, enum_type, containing_type, is_extension, extension_scope, options=..., file=..., has_default_value=..., containing_oneof=...): ...
    name = ...  # type: Any
    full_name = ...  # type: Any
    index = ...  # type: Any
    number = ...  # type: Any
    type = ...  # type: Any
    cpp_type = ...  # type: Any
    label = ...  # type: Any
    has_default_value = ...  # type: Any
    default_value = ...  # type: Any
    containing_type = ...  # type: Any
    message_type = ...  # type: Any
    enum_type = ...  # type: Any
    is_extension = ...  # type: Any
    extension_scope = ...  # type: Any
    containing_oneof = ...  # type: Any
    def __init__(self, name, full_name, index, number, type, cpp_type, label, default_value, message_type, enum_type, containing_type, is_extension, extension_scope, options=..., file=..., has_default_value=..., containing_oneof=...) -> None: ...
    @staticmethod
    def ProtoTypeToCppProtoType(proto_type): ...

class EnumDescriptor(_NestedDescriptorBase):
    def __new__(cls, name, full_name, filename, values, containing_type=..., options=..., file=..., serialized_start=..., serialized_end=...): ...
    values = ...  # type: Any
    values_by_name = ...  # type: Any
    values_by_number = ...  # type: Any
    def __init__(self, name, full_name, filename, values, containing_type=..., options=..., file=..., serialized_start=..., serialized_end=...) -> None: ...
    def CopyToProto(self, proto): ...

class EnumValueDescriptor(DescriptorBase):
    def __new__(cls, name, index, number, type=..., options=...): ...
    name = ...  # type: Any
    index = ...  # type: Any
    number = ...  # type: Any
    type = ...  # type: Any
    def __init__(self, name, index, number, type=..., options=...) -> None: ...

class OneofDescriptor:
    def __new__(cls, name, full_name, index, containing_type, fields): ...
    name = ...  # type: Any
    full_name = ...  # type: Any
    index = ...  # type: Any
    containing_type = ...  # type: Any
    fields = ...  # type: Any
    def __init__(self, name, full_name, index, containing_type, fields) -> None: ...

class ServiceDescriptor(_NestedDescriptorBase):
    index = ...  # type: Any
    methods = ...  # type: Any
    def __init__(self, name, full_name, index, methods, options=..., file=..., serialized_start=..., serialized_end=...) -> None: ...
    def FindMethodByName(self, name): ...
    def CopyToProto(self, proto): ...

class MethodDescriptor(DescriptorBase):
    name = ...  # type: Any
    full_name = ...  # type: Any
    index = ...  # type: Any
    containing_service = ...  # type: Any
    input_type = ...  # type: Any
    output_type = ...  # type: Any
    def __init__(self, name, full_name, index, containing_service, input_type, output_type, options=...) -> None: ...

class FileDescriptor(DescriptorBase):
    def __new__(cls, name, package, options=..., serialized_pb=..., dependencies=..., syntax=...): ...
    _options = ...  # type: Any
    message_types_by_name = ...  # type: Any
    name = ...  # type: Any
    package = ...  # type: Any
    syntax = ...  # type: Any
    serialized_pb = ...  # type: Any
    enum_types_by_name = ...  # type: Any
    extensions_by_name = ...  # type: Any
    dependencies = ...  # type: Any
    def __init__(self, name, package, options=..., serialized_pb=..., dependencies=..., syntax=...) -> None: ...
    def CopyToProto(self, proto): ...

def MakeDescriptor(desc_proto, package=..., build_file_if_cpp=..., syntax=...): ...
def _ParseOptions(message: Message, string: str) -> Message: ...



"""Test cases for AST diff (used for fine-grained incremental checking)"""

import os
from typing import List, Tuple, Dict, Optional

from mypy import build
from mypy.build import BuildSource
from mypy.defaults import PYTHON3_VERSION
from mypy.errors import CompileError
from mypy.nodes import MypyFile
from mypy.options import Options
from mypy.server.astdiff import snapshot_symbol_table, compare_symbol_table_snapshots
from mypy.test.config import test_temp_dir
from mypy.test.data import DataDrivenTestCase, DataSuite
from mypy.test.helpers import assert_string_arrays_equal


class ASTDiffSuite(DataSuite):
    files = ['diff.test']
    base_path = test_temp_dir
    optional_out = True

    def run_case(self, testcase: DataDrivenTestCase) -> None:
        first_src = '\n'.join(testcase.input)
        files_dict = dict(testcase.files)
        second_src = files_dict['tmp/next.py']

        messages1, files1 = self.build(first_src)
        messages2, files2 = self.build(second_src)

        a = []
        if messages1:
            a.extend(messages1)
        if messages2:
            a.append('== next ==')
            a.extend(messages2)

        assert files1 is not None and files2 is not None, ('cases where CompileError'
                                                           ' occurred should not be run')
        prefix = '__main__'
        snapshot1 = snapshot_symbol_table(prefix, files1['__main__'].names)
        snapshot2 = snapshot_symbol_table(prefix, files2['__main__'].names)
        diff = compare_symbol_table_snapshots(prefix, snapshot1, snapshot2)
        for trigger in sorted(diff):
            a.append(trigger)

        assert_string_arrays_equal(
            testcase.output, a,
            'Invalid output ({}, line {})'.format(testcase.file,
                                                  testcase.line))

    def build(self, source: str) -> Tuple[List[str], Optional[Dict[str, MypyFile]]]:
        options = Options()
        options.use_builtins_fixtures = True
        options.show_traceback = True
        options.cache_dir = os.devnull
        options.python_version = PYTHON3_VERSION
        try:
            result = build.build(sources=[BuildSource('main', None, source)],
                                 options=options,
                                 alt_lib_path=test_temp_dir)
        except CompileError as e:
            # TODO: Is it okay to return None?
            return e.messages, None
        return result.errors, result.files



from typing import Any, Callable, Dict, Generic, List, Optional, Sequence, Mapping, Tuple, Type, TypeVar, Union, overload
# `import X as X` is required to make these public
from . import exceptions as exceptions
from . import filters as filters
from . import converters as converters
from . import validators as validators

_T = TypeVar('_T')
_C = TypeVar('_C', bound=type)

_ValidatorType = Callable[[Any, Attribute, _T], Any]
_ConverterType = Callable[[Any], _T]
_FilterType = Callable[[Attribute, Any], bool]
# FIXME: in reality, if multiple validators are passed they must be in a list or tuple,
# but those are invariant and so would prevent subtypes of _ValidatorType from working
# when passed in a list or tuple.
_ValidatorArgType = Union[_ValidatorType[_T], Sequence[_ValidatorType[_T]]]

# _make --

NOTHING: object

# NOTE: Factory lies about its return type to make this possible: `x: List[int] = Factory(list)`
# Work around mypy issue #4554 by using an overload vs a Union.
@overload
def Factory(factory: Callable[[], _T], takes_self: bool = ...) -> _T: ...
@overload
def Factory(factory: Callable[[Any], _T], takes_self: bool = ...) -> _T: ...

class Attribute(Generic[_T]):
    name: str
    default: Optional[_T]
    validator: Optional[_ValidatorType[_T]]
    repr: bool
    cmp: bool
    hash: Optional[bool]
    init: bool
    converter: Optional[_ConverterType[_T]]
    metadata: Dict[Any, Any]
    type: Optional[Type[_T]]
    def __lt__(self, x: Attribute) -> bool: ...
    def __le__(self, x: Attribute) -> bool: ...
    def __gt__(self, x: Attribute) -> bool: ...
    def __ge__(self, x: Attribute) -> bool: ...


# NOTE: We had several choices for the annotation to use for type arg:
# 1) Type[_T]
#   - Pros: works in PyCharm without plugin support
#   - Cons: produces less informative error in the case of conflicting TypeVars
#     e.g. `attr.ib(default='bad', type=int)`
# 2) Callable[..., _T]
#   - Pros: more informative errors than #1
#   - Cons: validator tests results in confusing error.
#     e.g. `attr.ib(type=int, validator=validate_str)`
# 3) type (and do all of the work in the mypy plugin)
#   - Pros: in mypy, the behavior of type argument is exactly the same as with
#     annotations.
#   - Cons: completely disables type inspections in PyCharm when using the
#     type arg.
# We chose option #1 until either PyCharm adds support for attrs, or python 2
# reaches EOL.

# NOTE: If you update these, update `ib` and `attr` below.

# `attr` lies about its return type to make the following possible:
#     attr()    -> Any
#     attr(8)   -> int
#     attr(validator=<some callable>)  -> Whatever the callable expects.
# This makes this type of assignments possible:
#     x: int = attr(8)

# This form catches an explicit None or no default and infers the type from the other arguments.
@overload
def attrib(default: None = ...,
           validator: Optional[_ValidatorArgType[_T]] = ...,
           repr: bool = ...,
           cmp: bool = ...,
           hash: Optional[bool] = ...,
           init: bool = ...,
           convert: Optional[_ConverterType[_T]] = ...,
           metadata: Optional[Mapping[Any, Any]] = ...,
           type: Optional[Type[_T]] = ...,
           converter: Optional[_ConverterType[_T]] = ...) -> _T: ...
# This form catches an explicit default argument.
@overload
def attrib(default: _T,
           validator: Optional[_ValidatorArgType[_T]] = ...,
           repr: bool = ...,
           cmp: bool = ...,
           hash: Optional[bool] = ...,
           init: bool = ...,
           convert: Optional[_ConverterType[_T]] = ...,
           metadata: Optional[Mapping[Any, Any]] = ...,
           type: Optional[Type[_T]] = ...,
           converter: Optional[_ConverterType[_T]] = ...) -> _T: ...
# This form catches explicit None or no default but with no other arguments returns Any.
@overload
def attrib(default: None = ...,
           validator: None = ...,
           repr: bool = ...,
           cmp: bool = ...,
           hash: Optional[bool] = ...,
           init: bool = ...,
           convert: None = ...,
           metadata: Optional[Mapping[Any, Any]] = ...,
           type: None = ...,
           converter: None = ...) -> Any: ...
# This form covers type=non-Type: e.g. forward references (str), Any
@overload
def attrib(default: Optional[_T] = ...,
           validator: Optional[_ValidatorArgType[_T]] = ...,
           repr: bool = ...,
           cmp: bool = ...,
           hash: Optional[bool] = ...,
           init: bool = ...,
           convert: Optional[_ConverterType[_T]] = ...,
           metadata: Optional[Mapping[Any, Any]] = ...,
           type: object = ...,
           converter: Optional[_ConverterType[_T]] = ...) -> Any: ...


# NOTE: If you update these, update `s` and `attributes` below.
@overload
def attrs(maybe_cls: _C,
          these: Optional[Dict[str, Any]] = ...,
          repr_ns: Optional[str] = ...,
          repr: bool = ...,
          cmp: bool = ...,
          hash: Optional[bool] = ...,
          init: bool = ...,
          slots: bool = ...,
          frozen: bool = ...,
          str: bool = ...,
          auto_attribs: bool = ...) -> _C: ...
@overload
def attrs(maybe_cls: None = ...,
          these: Optional[Dict[str, Any]] = ...,
          repr_ns: Optional[str] = ...,
          repr: bool = ...,
          cmp: bool = ...,
          hash: Optional[bool] = ...,
          init: bool = ...,
          slots: bool = ...,
          frozen: bool = ...,
          str: bool = ...,
          auto_attribs: bool = ...) -> Callable[[_C], _C]: ...


# TODO: add support for returning NamedTuple from the mypy plugin
class _Fields(Tuple[Attribute, ...]):
    def __getattr__(self, name: str) -> Attribute: ...

def fields(cls: type) -> _Fields: ...
def validate(inst: Any) -> None: ...

# TODO: add support for returning a proper attrs class from the mypy plugin
# we use Any instead of _CountingAttr so that e.g. `make_class('Foo', [attr.ib()])` is valid
def make_class(name: str,
               attrs: Union[List[str], Tuple[str, ...], Dict[str, Any]],
               bases: Tuple[type, ...] = ...,
               repr_ns: Optional[str] = ...,
               repr: bool = ...,
               cmp: bool = ...,
               hash: Optional[bool] = ...,
               init: bool = ...,
               slots: bool = ...,
               frozen: bool = ...,
               str: bool = ...,
               auto_attribs: bool = ...) -> type: ...

# _funcs --

# TODO: add support for returning TypedDict from the mypy plugin
# FIXME: asdict/astuple do not honor their factory args.  waiting on one of these:
# https://github.com/python/mypy/issues/4236
# https://github.com/python/typing/issues/253
def asdict(inst: Any,
           recurse: bool = ...,
           filter: Optional[_FilterType] = ...,
           dict_factory: Type[Mapping[Any, Any]] = ...,
           retain_collection_types: bool = ...) -> Dict[str, Any]: ...
# TODO: add support for returning NamedTuple from the mypy plugin
def astuple(inst: Any,
            recurse: bool = ...,
            filter: Optional[_FilterType] = ...,
            tuple_factory: Type[Sequence] = ...,
            retain_collection_types: bool = ...) -> Tuple[Any, ...]: ...
def has(cls: type) -> bool: ...
def assoc(inst: _T, **changes: Any) -> _T: ...
def evolve(inst: _T, **changes: Any) -> _T: ...

# _config --

def set_run_validators(run: bool) -> None: ...
def get_run_validators() -> bool: ...


# aliases --

# FIXME: there is a bug in PyCharm with creating aliases to overloads.
# Use the aliases instead of the duplicated overloads when the bug is fixed:
# https://youtrack.jetbrains.com/issue/PY-27788

# s = attributes = attrs
# ib = attr = attrib
# dataclass = attrs # Technically, partial(attrs, auto_attribs=True) ;)


@overload
def ib(default: None = ...,
       validator: Optional[_ValidatorArgType[_T]] = ...,
       repr: bool = ...,
       cmp: bool = ...,
       hash: Optional[bool] = ...,
       init: bool = ...,
       convert: Optional[_ConverterType[_T]] = ...,
       metadata: Optional[Mapping[Any, Any]] = ...,
       type: Optional[Type[_T]] = ...,
       converter: Optional[_ConverterType[_T]] = ...) -> _T: ...
@overload
def ib(default: _T,
       validator: Optional[_ValidatorArgType[_T]] = ...,
       repr: bool = ...,
       cmp: bool = ...,
       hash: Optional[bool] = ...,
       init: bool = ...,
       convert: Optional[_ConverterType[_T]] = ...,
       metadata: Optional[Mapping[Any, Any]] = ...,
       type: Optional[Type[_T]] = ...,
       converter: Optional[_ConverterType[_T]] = ...) -> _T: ...
@overload
def ib(default: None = ...,
       validator: None = ...,
       repr: bool = ...,
       cmp: bool = ...,
       hash: Optional[bool] = ...,
       init: bool = ...,
       convert: None = ...,
       metadata: Optional[Mapping[Any, Any]] = ...,
       type: None = ...,
       converter: None = ...) -> Any: ...
@overload
def ib(default: Optional[_T] = ...,
       validator: Optional[_ValidatorArgType[_T]] = ...,
       repr: bool = ...,
       cmp: bool = ...,
       hash: Optional[bool] = ...,
       init: bool = ...,
       convert: Optional[_ConverterType[_T]] = ...,
       metadata: Optional[Mapping[Any, Any]] = ...,
       type: object = ...,
       converter: Optional[_ConverterType[_T]] = ...) -> Any: ...

@overload
def attr(default: None = ...,
         validator: Optional[_ValidatorArgType[_T]] = ...,
         repr: bool = ...,
         cmp: bool = ...,
         hash: Optional[bool] = ...,
         init: bool = ...,
         convert: Optional[_ConverterType[_T]] = ...,
         metadata: Optional[Mapping[Any, Any]] = ...,
         type: Optional[Type[_T]] = ...,
         converter: Optional[_ConverterType[_T]] = ...) -> _T: ...
@overload
def attr(default: _T,
         validator: Optional[_ValidatorArgType[_T]] = ...,
         repr: bool = ...,
         cmp: bool = ...,
         hash: Optional[bool] = ...,
         init: bool = ...,
         convert: Optional[_ConverterType[_T]] = ...,
         metadata: Optional[Mapping[Any, Any]] = ...,
         type: Optional[Type[_T]] = ...,
         converter: Optional[_ConverterType[_T]] = ...) -> _T: ...
@overload
def attr(default: None = ...,
         validator: None = ...,
         repr: bool = ...,
         cmp: bool = ...,
         hash: Optional[bool] = ...,
         init: bool = ...,
         convert: None = ...,
         metadata: Optional[Mapping[Any, Any]] = ...,
         type: None = ...,
         converter: None = ...) -> Any: ...
@overload
def attr(default: Optional[_T] = ...,
         validator: Optional[_ValidatorArgType[_T]] = ...,
         repr: bool = ...,
         cmp: bool = ...,
         hash: Optional[bool] = ...,
         init: bool = ...,
         convert: Optional[_ConverterType[_T]] = ...,
         metadata: Optional[Mapping[Any, Any]] = ...,
         type: object = ...,
         converter: Optional[_ConverterType[_T]] = ...) -> Any: ...

@overload
def attributes(maybe_cls: _C,
               these: Optional[Dict[str, Any]] = ...,
               repr_ns: Optional[str] = ...,
               repr: bool = ...,
               cmp: bool = ...,
               hash: Optional[bool] = ...,
               init: bool = ...,
               slots: bool = ...,
               frozen: bool = ...,
               str: bool = ...,
               auto_attribs: bool = ...) -> _C: ...
@overload
def attributes(maybe_cls: None = ...,
               these: Optional[Dict[str, Any]] = ...,
               repr_ns: Optional[str] = ...,
               repr: bool = ...,
               cmp: bool = ...,
               hash: Optional[bool] = ...,
               init: bool = ...,
               slots: bool = ...,
               frozen: bool = ...,
               str: bool = ...,
               auto_attribs: bool = ...) -> Callable[[_C], _C]: ...

@overload
def s(maybe_cls: _C,
      these: Optional[Dict[str, Any]] = ...,
      repr_ns: Optional[str] = ...,
      repr: bool = ...,
      cmp: bool = ...,
      hash: Optional[bool] = ...,
      init: bool = ...,
      slots: bool = ...,
      frozen: bool = ...,
      str: bool = ...,
      auto_attribs: bool = ...) -> _C: ...
@overload
def s(maybe_cls: None = ...,
      these: Optional[Dict[str, Any]] = ...,
      repr_ns: Optional[str] = ...,
      repr: bool = ...,
      cmp: bool = ...,
      hash: Optional[bool] = ...,
      init: bool = ...,
      slots: bool = ...,
      frozen: bool = ...,
      str: bool = ...,
      auto_attribs: bool = ...) -> Callable[[_C], _C]: ...

# same as above, but with auto_attrib=True
@overload
def dataclass(maybe_cls: _C,
              these: Optional[Dict[str, Any]] = ...,
              repr_ns: Optional[str] = ...,
              repr: bool = ...,
              cmp: bool = ...,
              hash: Optional[bool] = ...,
              init: bool = ...,
              slots: bool = ...,
              frozen: bool = ...,
              str: bool = ...) -> _C: ...
@overload
def dataclass(maybe_cls: None = ...,
              these: Optional[Dict[str, Any]] = ...,
              repr_ns: Optional[str] = ...,
              repr: bool = ...,
              cmp: bool = ...,
              hash: Optional[bool] = ...,
              init: bool = ...,
              slots: bool = ...,
              frozen: bool = ...,
              str: bool = ...) -> Callable[[_C], _C]: ...



from typing import Any, Dict, Iterable, Optional, Type

from .message import Message
from .descriptor import Descriptor
from .descriptor_pool import DescriptorPool

class MessageFactory:
    pool = ...  # type: Any
    def __init__(self, pool: Optional[DescriptorPool] = ...) -> None: ...
    def GetPrototype(self, descriptor: Descriptor) -> Type[Message]: ...
    def GetMessages(self, files: Iterable[str]) -> Dict[str, Type[Message]]: ...

def GetMessages(file_protos: Iterable[str]) -> Dict[str, Type[Message]]: ...



# Stubs for operator

from typing import (
    Any, Callable, Container, Mapping, MutableMapping, MutableSequence, Sequence, SupportsAbs, Tuple,
    TypeVar, overload,
)
import sys


_T = TypeVar('_T')
_K = TypeVar('_K')
_V = TypeVar('_V')


def lt(a: Any, b: Any) -> Any: ...
def le(a: Any, b: Any) -> Any: ...
def eq(a: Any, b: Any) -> Any: ...
def ne(a: Any, b: Any) -> Any: ...
def ge(a: Any, b: Any) -> Any: ...
def gt(a: Any, b: Any) -> Any: ...
def __lt__(a: Any, b: Any) -> Any: ...
def __le__(a: Any, b: Any) -> Any: ...
def __eq__(a: Any, b: Any) -> Any: ...
def __ne__(a: Any, b: Any) -> Any: ...
def __ge__(a: Any, b: Any) -> Any: ...
def __gt__(a: Any, b: Any) -> Any: ...

def not_(obj: Any) -> bool: ...
def __not__(obj: Any) -> bool: ...

def truth(x: Any) -> bool: ...

def is_(a: Any, b: Any) -> bool: ...

def is_not(a: Any, b: Any) -> bool: ...

def abs(x: SupportsAbs) -> Any: ...
def __abs__(a: SupportsAbs) -> Any: ...

def add(a: Any, b: Any) -> Any: ...
def __add__(a: Any, b: Any) -> Any: ...

def and_(a: Any, b: Any) -> Any: ...
def __and__(a: Any, b: Any) -> Any: ...

if sys.version_info < (3, ):
    def div(a: Any, b: Any) -> Any: ...
    def __div__(a: Any, b: Any) -> Any: ...

def floordiv(a: Any, b: Any) -> Any: ...
def __floordiv__(a: Any, b: Any) -> Any: ...

def index(a: Any) -> int: ...
def __index__(a: Any) -> int: ...

def inv(obj: Any) -> Any: ...
def invert(obj: Any) -> Any: ...
def __inv__(obj: Any) -> Any: ...
def __invert__(obj: Any) -> Any: ...

def lshift(a: Any, b: Any) -> Any: ...
def __lshift__(a: Any, b: Any) -> Any: ...

def mod(a: Any, b: Any) -> Any: ...
def __mod__(a: Any, b: Any) -> Any: ...

def mul(a: Any, b: Any) -> Any: ...
def __mul__(a: Any, b: Any) -> Any: ...

if sys.version_info >= (3, 5):
    def matmul(a: Any, b: Any) -> Any: ...
    def __matmul__(a: Any, b: Any) -> Any: ...

def neg(obj: Any) -> Any: ...
def __neg__(obj: Any) -> Any: ...

def or_(a: Any, b: Any) -> Any: ...
def __or__(a: Any, b: Any) -> Any: ...

def pos(obj: Any) -> Any: ...
def __pos__(obj: Any) -> Any: ...

def pow(a: Any, b: Any) -> Any: ...
def __pow__(a: Any, b: Any) -> Any: ...

def rshift(a: Any, b: Any) -> Any: ...
def __rshift__(a: Any, b: Any) -> Any: ...

def sub(a: Any, b: Any) -> Any: ...
def __sub__(a: Any, b: Any) -> Any: ...

def truediv(a: Any, b: Any) -> Any: ...
def __truediv__(a: Any, b: Any) -> Any: ...

def xor(a: Any, b: Any) -> Any: ...
def __xor__(a: Any, b: Any) -> Any: ...

def concat(a: Sequence[_T], b: Sequence[_T]) -> Sequence[_T]: ...
def __concat__(a: Sequence[_T], b: Sequence[_T]) -> Sequence[_T]: ...

def contains(a: Container[Any], b: Any) -> bool: ...
def __contains__(a: Container[Any], b: Any) -> bool: ...

def countOf(a: Container[Any], b: Any) -> int: ...

@overload
def delitem(a: MutableSequence[_T], b: int) -> None: ...
@overload
def delitem(a: MutableMapping[_K, _V], b: _K) -> None: ...
@overload
def __delitem__(a: MutableSequence[_T], b: int) -> None: ...
@overload
def __delitem__(a: MutableMapping[_K, _V], b: _K) -> None: ...

if sys.version_info < (3, ):
    def delslice(a: MutableSequence[Any], b: int, c: int) -> None: ...
    def __delslice__(a: MutableSequence[Any], b: int, c: int) -> None: ...

@overload
def getitem(a: Sequence[_T], b: int) -> _T: ...
@overload
def getitem(a: Mapping[_K, _V], b: _K) -> _V: ...
@overload
def __getitem__(a: Sequence[_T], b: int) -> _T: ...
@overload
def __getitem__(a: Mapping[_K, _V], b: _K) -> _V: ...

if sys.version_info < (3, ):
    def getslice(a: Sequence[_T], b: int, c: int) -> Sequence[_T]: ...
    def __getslice__(a: Sequence[_T], b: int, c: int) -> Sequence[_T]: ...

def indexOf(a: Sequence[_T], b: _T) -> int: ...

if sys.version_info < (3, ):
    def repeat(a: Any, b: int) -> Any: ...
    def __repeat__(a: Any, b: int) -> Any: ...

if sys.version_info < (3, ):
    def sequenceIncludes(a: Container[Any], b: Any) -> bool: ...

@overload
def setitem(a: MutableSequence[_T], b: int, c: _T) -> None: ...
@overload
def setitem(a: MutableMapping[_K, _V], b: _K, c: _V) -> None: ...
@overload
def __setitem__(a: MutableSequence[_T], b: int, c: _T) -> None: ...
@overload
def __setitem__(a: MutableMapping[_K, _V], b: _K, c: _V) -> None: ...

if sys.version_info < (3, ):
    def setslice(a: MutableSequence[_T], b: int, c: int, v: Sequence[_T]) -> None: ...
    def __setslice__(a: MutableSequence[_T], b: int, c: int, v: Sequence[_T]) -> None: ...


if sys.version_info >= (3, 4):
    def length_hint(obj: Any, default: int = ...) -> int: ...

@overload
def attrgetter(attr: str) -> Callable[[Any], Any]: ...
@overload
def attrgetter(*attrs: str) -> Callable[[Any], Tuple[Any, ...]]: ...

@overload
def itemgetter(item: Any) -> Callable[[Any], Any]: ...
@overload
def itemgetter(*items: Any) -> Callable[[Any], Tuple[Any, ...]]: ...

def methodcaller(name: str, *args: Any, **kwargs: Any) -> Callable[..., Any]: ...


def iadd(a: Any, b: Any) -> Any: ...
def __iadd__(a: Any, b: Any) -> Any: ...

def iand(a: Any, b: Any) -> Any: ...
def __iand__(a: Any, b: Any) -> Any: ...

def iconcat(a: Any, b: Any) -> Any: ...
def __iconcat__(a: Any, b: Any) -> Any: ...

if sys.version_info < (3, ):
    def idiv(a: Any, b: Any) -> Any: ...
    def __idiv__(a: Any, b: Any) -> Any: ...

def ifloordiv(a: Any, b: Any) -> Any: ...
def __ifloordiv__(a: Any, b: Any) -> Any: ...

def ilshift(a: Any, b: Any) -> Any: ...
def __ilshift__(a: Any, b: Any) -> Any: ...

def imod(a: Any, b: Any) -> Any: ...
def __imod__(a: Any, b: Any) -> Any: ...

def imul(a: Any, b: Any) -> Any: ...
def __imul__(a: Any, b: Any) -> Any: ...

if sys.version_info >= (3, 5):
    def imatmul(a: Any, b: Any) -> Any: ...
    def __imatmul__(a: Any, b: Any) -> Any: ...

def ior(a: Any, b: Any) -> Any: ...
def __ior__(a: Any, b: Any) -> Any: ...

def ipow(a: Any, b: Any) -> Any: ...
def __ipow__(a: Any, b: Any) -> Any: ...

if sys.version_info < (3, ):
    def irepeat(a: Any, b: int) -> Any: ...
    def __irepeat__(a: Any, b: int) -> Any: ...

def irshift(a: Any, b: Any) -> Any: ...
def __irshift__(a: Any, b: Any) -> Any: ...

def isub(a: Any, b: Any) -> Any: ...
def __isub__(a: Any, b: Any) -> Any: ...

def itruediv(a: Any, b: Any) -> Any: ...
def __itruediv__(a: Any, b: Any) -> Any: ...

def ixor(a: Any, b: Any) -> Any: ...
def __ixor__(a: Any, b: Any) -> Any: ...


if sys.version_info < (3, ):
    def isCallable(x: Any) -> bool: ...
    def isMappingType(x: Any) -> bool: ...
    def isNumberType(x: Any) -> bool: ...
    def isSequenceType(x: Any) -> bool: ...



import sys
from typing import Any, Callable, Generic, Optional, TypeVar

_T = TypeVar('_T')

class CallableProxyType(object):  # "weakcallableproxy"
    def __getattr__(self, attr: str) -> Any: ...

class ProxyType(object):  # "weakproxy"
    def __getattr__(self, attr: str) -> Any: ...

class ReferenceType(Generic[_T]):
    if sys.version_info >= (3, 4):
        __callback__: Callable[[ReferenceType[_T]], Any]
    def __init__(self, o: _T, callback: Callable[[ReferenceType[_T]],
                                                 Any] = ...) -> None: ...
    def __call__(self) -> Optional[_T]: ...
    def __hash__(self) -> int: ...

ref = ReferenceType

def getweakrefcount(object: Any) -> int: ...
def getweakrefs(object: Any) -> int: ...
def proxy(object: _T, callback: Callable[[_T], Any] = ...) -> ref[_T]: ...



PYTHON2_VERSION = (2, 7)
PYTHON3_VERSION = (3, 6)
PYTHON3_VERSION_MIN = (3, 3)
CACHE_DIR = '.mypy_cache'
CONFIG_FILE = 'mypy.ini'
SHARED_CONFIG_FILES = ('setup.cfg',)
USER_CONFIG_FILES = ('~/.mypy.ini',)
CONFIG_FILES = (CONFIG_FILE,) + SHARED_CONFIG_FILES + USER_CONFIG_FILES



import importlib.abc
import sys
import types
from typing import Any, Callable, List, Optional, Sequence, Tuple, Union

# ModuleSpec is exported from this module, but for circular import
# reasons exists in its own stub file (with Loader and ModuleType).
if sys.version_info >= (3, 4):
    from _importlib_modulespec import ModuleSpec as ModuleSpec  # Exported

if sys.version_info >= (3, 3):
    class BuiltinImporter(importlib.abc.MetaPathFinder,
                          importlib.abc.InspectLoader):
        # MetaPathFinder
        @classmethod
        def find_module(
            cls, fullname: str,
            path: Optional[Sequence[importlib.abc._Path]]
        ) -> Optional[importlib.abc.Loader]:
            ...
        if sys.version_info >= (3, 4):
            @classmethod
            def find_spec(cls, fullname: str,
                          path: Optional[Sequence[importlib.abc._Path]],
                          target: Optional[types.ModuleType] = ...) -> Optional[ModuleSpec]:
                ...
        # InspectLoader
        @classmethod
        def is_package(cls, fullname: str) -> bool: ...
        @classmethod
        def load_module(cls, fullname: str) -> types.ModuleType: ...
        @classmethod
        def get_code(cls, fullname: str) -> None: ...
        @classmethod
        def get_source(cls, fullname: str) -> None: ...
        # Loader
        @classmethod
        def load_module(cls, fullname: str) -> types.ModuleType: ...
        if sys.version_info >= (3, 3):
            @staticmethod
            def module_repr(module: types.ModuleType) -> str: ...  # type: ignore
        if sys.version_info >= (3, 4):
            @classmethod
            def create_module(cls, spec: ModuleSpec) -> Optional[types.ModuleType]: ...
            @classmethod
            def exec_module(cls, module: types.ModuleType) -> None: ...
else:
    class BuiltinImporter(importlib.abc.InspectLoader):
        # MetaPathFinder
        @classmethod
        def find_module(
            cls, fullname: str,
            path: Optional[Sequence[importlib.abc._Path]]
        ) -> Optional[importlib.abc.Loader]:
            ...
        # InspectLoader
        @classmethod
        def is_package(cls, fullname: str) -> bool: ...
        @classmethod
        def load_module(cls, fullname: str) -> types.ModuleType: ...
        @classmethod
        def get_code(cls, fullname: str) -> None: ...
        @classmethod
        def get_source(cls, fullname: str) -> None: ...
        # Loader
        @classmethod
        def load_module(cls, fullname: str) -> types.ModuleType: ...

if sys.version_info >= (3, 3):
    class FrozenImporter(importlib.abc.MetaPathFinder, importlib.abc.InspectLoader):
        # MetaPathFinder
        @classmethod
        def find_module(
            cls, fullname: str,
            path: Optional[Sequence[importlib.abc._Path]]
        ) -> Optional[importlib.abc.Loader]:
            ...
        if sys.version_info >= (3, 4):
            @classmethod
            def find_spec(cls, fullname: str,
                          path: Optional[Sequence[importlib.abc._Path]],
                          target: Optional[types.ModuleType] = ...) -> Optional[ModuleSpec]:
                ...
        # InspectLoader
        @classmethod
        def is_package(cls, fullname: str) -> bool: ...
        @classmethod
        def load_module(cls, fullname: str) -> types.ModuleType: ...
        @classmethod
        def get_code(cls, fullname: str) -> None: ...
        @classmethod
        def get_source(cls, fullname: str) -> None: ...
        # Loader
        @classmethod
        def load_module(cls, fullname: str) -> types.ModuleType: ...
        if sys.version_info >= (3, 3):
            @staticmethod
            def module_repr(module: types.ModuleType) -> str: ...  # type: ignore
        if sys.version_info >= (3, 4):
            @classmethod
            def create_module(cls, spec: ModuleSpec) -> Optional[types.ModuleType]:
                ...
            @staticmethod
            def exec_module(module: types.ModuleType) -> None: ...  # type: ignore
else:
    class FrozenImporter(importlib.abc.InspectLoader):
        # MetaPathFinder
        @classmethod
        def find_module(
            cls, fullname: str,
            path: Optional[Sequence[importlib.abc._Path]]
        ) -> Optional[importlib.abc.Loader]:
            ...
        # InspectLoader
        @classmethod
        def is_package(cls, fullname: str) -> bool: ...
        @classmethod
        def load_module(cls, fullname: str) -> types.ModuleType: ...
        @classmethod
        def get_code(cls, fullname: str) -> None: ...  # type: ignore
        @classmethod
        def get_source(cls, fullname: str) -> None: ...  # type: ignore
        # Loader
        @classmethod
        def load_module(cls, fullname: str) -> types.ModuleType: ...

if sys.version_info >= (3, 3):
    class WindowsRegistryFinder(importlib.abc.MetaPathFinder):
        @classmethod
        def find_module(
            cls, fullname: str,
            path: Optional[Sequence[importlib.abc._Path]]
        ) -> Optional[importlib.abc.Loader]:
            ...
        if sys.version_info >= (3, 4):
            @classmethod
            def find_spec(cls, fullname: str,
                          path: Optional[Sequence[importlib.abc._Path]],
                          target: Optional[types.ModuleType] = ...) -> Optional[ModuleSpec]:
                ...
else:
    class WindowsRegisteryFinder:
        @classmethod
        def find_module(
            cls, fullname: str,
            path: Optional[Sequence[importlib.abc._Path]]
        ) -> Optional[importlib.abc.Loader]:
            ...

if sys.version_info >= (3, 3):
    class PathFinder(importlib.abc.MetaPathFinder): ...
else:
    class PathFinder: ...

if sys.version_info >= (3, 3):
    SOURCE_SUFFIXES = ...  # type: List[str]
    DEBUG_BYTECODE_SUFFIXES = ...  # type: List[str]
    OPTIMIZED_BYTECODE_SUFFIXES = ...  # type: List[str]
    BYTECODE_SUFFIXES = ...  # type: List[str]
    EXTENSION_SUFFIXES = ...  # type: List[str]

    def all_suffixes() -> List[str]: ...

    class FileFinder(importlib.abc.PathEntryFinder):
        path = ...  # type: str
        def __init__(
            self, path: str,
            *loader_details: Tuple[importlib.abc.Loader, List[str]]
        ) -> None: ...
        @classmethod
        def path_hook(
            *loader_details: Tuple[importlib.abc.Loader, List[str]]
        ) -> Callable[[str], importlib.abc.PathEntryFinder]: ...

    class SourceFileLoader(importlib.abc.FileLoader,
                           importlib.abc.SourceLoader):
        ...

    class SourcelessFileLoader(importlib.abc.FileLoader,
                               importlib.abc.SourceLoader):
        ...

    class ExtensionFileLoader(importlib.abc.ExecutionLoader):
        def get_filename(self, fullname: str) -> importlib.abc._Path: ...
        def get_source(self, fullname: str) -> None: ...  # type: ignore



# Automatically generated by pytype, manually fixed up. May still contain errors.

from typing import Any, Callable, Dict, Generator, Iterator, List, Tuple, Union, Iterable

__all__ = ...  # type: List[str]
__author__ = ...  # type: str
__credits__ = ...  # type: str

AMPER = ...  # type: int
AMPEREQUAL = ...  # type: int
AT = ...  # type: int
BACKQUOTE = ...  # type: int
Binnumber = ...  # type: str
Bracket = ...  # type: str
CIRCUMFLEX = ...  # type: int
CIRCUMFLEXEQUAL = ...  # type: int
COLON = ...  # type: int
COMMA = ...  # type: int
COMMENT = ...  # type: int
Comment = ...  # type: str
ContStr = ...  # type: str
DEDENT = ...  # type: int
DOT = ...  # type: int
DOUBLESLASH = ...  # type: int
DOUBLESLASHEQUAL = ...  # type: int
DOUBLESTAR = ...  # type: int
DOUBLESTAREQUAL = ...  # type: int
Decnumber = ...  # type: str
Double = ...  # type: str
Double3 = ...  # type: str
ENDMARKER = ...  # type: int
EQEQUAL = ...  # type: int
EQUAL = ...  # type: int
ERRORTOKEN = ...  # type: int
Expfloat = ...  # type: str
Exponent = ...  # type: str
Floatnumber = ...  # type: str
Funny = ...  # type: str
GREATER = ...  # type: int
GREATEREQUAL = ...  # type: int
Hexnumber = ...  # type: str
INDENT = ...  # type: int

def ISEOF(x: int) -> bool: ...
def ISNONTERMINAL(x: int) -> bool: ...
def ISTERMINAL(x: int) -> bool: ...

Ignore = ...  # type: str
Imagnumber = ...  # type: str
Intnumber = ...  # type: str
LBRACE = ...  # type: int
LEFTSHIFT = ...  # type: int
LEFTSHIFTEQUAL = ...  # type: int
LESS = ...  # type: int
LESSEQUAL = ...  # type: int
LPAR = ...  # type: int
LSQB = ...  # type: int
MINEQUAL = ...  # type: int
MINUS = ...  # type: int
NAME = ...  # type: int
NEWLINE = ...  # type: int
NL = ...  # type: int
NOTEQUAL = ...  # type: int
NT_OFFSET = ...  # type: int
NUMBER = ...  # type: int
N_TOKENS = ...  # type: int
Name = ...  # type: str
Number = ...  # type: str
OP = ...  # type: int
Octnumber = ...  # type: str
Operator = ...  # type: str
PERCENT = ...  # type: int
PERCENTEQUAL = ...  # type: int
PLUS = ...  # type: int
PLUSEQUAL = ...  # type: int
PlainToken = ...  # type: str
Pointfloat = ...  # type: str
PseudoExtras = ...  # type: str
PseudoToken = ...  # type: str
RBRACE = ...  # type: int
RIGHTSHIFT = ...  # type: int
RIGHTSHIFTEQUAL = ...  # type: int
RPAR = ...  # type: int
RSQB = ...  # type: int
SEMI = ...  # type: int
SLASH = ...  # type: int
SLASHEQUAL = ...  # type: int
STAR = ...  # type: int
STAREQUAL = ...  # type: int
STRING = ...  # type: int
Single = ...  # type: str
Single3 = ...  # type: str
Special = ...  # type: str
String = ...  # type: str
TILDE = ...  # type: int
Token = ...  # type: str
Triple = ...  # type: str
VBAR = ...  # type: int
VBAREQUAL = ...  # type: int
Whitespace = ...  # type: str
chain = ...  # type: type
double3prog = ...  # type: type
endprogs = ...  # type: Dict[str, Any]
pseudoprog = ...  # type: type
single3prog = ...  # type: type
single_quoted = ...  # type: Dict[str, str]
t = ...  # type: str
tabsize = ...  # type: int
tok_name = ...  # type: Dict[int, str]
tokenprog = ...  # type: type
triple_quoted = ...  # type: Dict[str, str]
x = ...  # type: str

_Pos = Tuple[int, int]
_TokenType = Tuple[int, str, _Pos, _Pos, str]

def any(*args, **kwargs) -> str: ...
def generate_tokens(readline: Callable[[], str]) -> Generator[_TokenType, None, None]: ...
def group(*args: str) -> str: ...
def maybe(*args: str) -> str: ...
def printtoken(type: int, token: str, srow_scol: _Pos, erow_ecol: _Pos, line: str) -> None: ...
def tokenize(readline: Callable[[], str], tokeneater: Callable[[Tuple[int, str, _Pos, _Pos, str]], None]) -> None: ...
def tokenize_loop(readline: Callable[[], str], tokeneater: Callable[[Tuple[int, str, _Pos, _Pos, str]], None]) -> None: ...
def untokenize(iterable: Iterable[_TokenType]) -> str: ...

class StopTokenizing(Exception):
    pass

class TokenError(Exception):
    pass

class Untokenizer:
    prev_col = ...  # type: int
    prev_row = ...  # type: int
    tokens = ...  # type: List[str]
    def __init__(self) -> None: ...
    def add_whitespace(self, _Pos) -> None: ...
    def compat(self, token: Tuple[int, Any], iterable: Iterator[_TokenType]) -> None: ...
    def untokenize(self, iterable: Iterable[_TokenType]) -> str: ...



# Type declaration for a WSGI Function in Python 3
#
# wsgiref/types.py doesn't exist and neither does WSGIApplication, it's a type
# provided for type checking purposes.
#
# This means you cannot simply import wsgiref.types in your code. Instead,
# use the `TYPE_CHECKING` flag from the typing module:
#
#   from typing import TYPE_CHECKING
#
#   if TYPE_CHECKING:
#       from wsgiref.types import WSGIApplication
#
# This import is now only taken into account by the type checker. Consequently,
# you need to use 'WSGIApplication' and not simply WSGIApplication when type
# hinting your code.  Otherwise Python will raise NameErrors.

from typing import Callable, Dict, Iterable, List, Optional, Tuple, Type, Union, Any
from types import TracebackType

_exc_info = Tuple[Optional[Type[BaseException]],
                  Optional[BaseException],
                  Optional[TracebackType]]
WSGIEnvironment = Dict[str, Any]
WSGIApplication = Callable[
    [
        WSGIEnvironment,
        Union[
            Callable[[str, List[Tuple[str, str]]], Callable[[Union[bytes, str]], None]],
            Callable[[str, List[Tuple[str, str]], _exc_info], Callable[[Union[bytes, str]], None]]
        ]
    ],
    Iterable[Union[bytes, str]]
]



from typing import Any, Optional, List, Type

import threading
import sys
import weakref
import array
import itertools

from multiprocessing import TimeoutError, cpu_count
from multiprocessing.dummy.connection import Pipe
from threading import Lock, RLock, Semaphore, BoundedSemaphore
from threading import Event
from Queue import Queue

__all__ = ...  # type: List[str]


class DummyProcess(threading.Thread):
    _children = ...  # type: weakref.WeakKeyDictionary
    _parent = ...  # type: threading.Thread
    _pid = ...  # type: None
    _start_called = ...  # type: bool
    def __init__(self, group=..., target=..., name=..., args=..., kwargs=...) -> None: ...
    @property
    def exitcode(self) -> Optional[int]: ...


Process = DummyProcess

# This should be threading._Condition but threading.pyi exports it as Condition
class Condition(threading.Condition):
    notify_all = ...  # type: Any

class Namespace(object):
    def __init__(self, **kwds) -> None: ...

class Value(object):
    _typecode = ...  # type: Any
    _value = ...  # type: Any
    value = ...  # type: Any
    def __init__(self, typecode, value, lock=...) -> None: ...
    def _get(self) -> Any: ...
    def _set(self, value) -> None: ...

JoinableQueue = Queue

def Array(typecode, sequence, lock=...) -> array.array: ...
def Manager() -> Any: ...
def Pool(processes=..., initializer=..., initargs=...) -> Any: ...
def active_children() -> List: ...
def current_process() -> threading.Thread: ...
def freeze_support() -> None: ...
def shutdown() -> None: ...



from typing import Iterator, List, Set, TypeVar
from collections import Mapping

_T = TypeVar('_T')
_KT = TypeVar('_KT')
_VT = TypeVar('_VT')

class LazyDict(Mapping[_KT, _VT]):
    def __getitem__(self, key: _KT) -> _VT: ...
    def __iter__(self) -> Iterator[_KT]: ...
    def __len__(self) -> int: ...

class LazyList(List[_T]): ...
class LazySet(Set[_T]): ...



# Stubs for six.moves.urllib.request
#
# Note: Commented out items means they weren't implemented at the time.
# Uncomment them when the modules have been added to the typeshed.
from urllib.request import urlopen as urlopen
from urllib.request import install_opener as install_opener
from urllib.request import build_opener as build_opener
from urllib.request import pathname2url as pathname2url
from urllib.request import url2pathname as url2pathname
from urllib.request import getproxies as getproxies
from urllib.request import Request as Request
from urllib.request import OpenerDirector as OpenerDirector
from urllib.request import HTTPDefaultErrorHandler as HTTPDefaultErrorHandler
from urllib.request import HTTPRedirectHandler as HTTPRedirectHandler
from urllib.request import HTTPCookieProcessor as HTTPCookieProcessor
from urllib.request import ProxyHandler as ProxyHandler
from urllib.request import BaseHandler as BaseHandler
from urllib.request import HTTPPasswordMgr as HTTPPasswordMgr
from urllib.request import HTTPPasswordMgrWithDefaultRealm as HTTPPasswordMgrWithDefaultRealm
from urllib.request import AbstractBasicAuthHandler as AbstractBasicAuthHandler
from urllib.request import HTTPBasicAuthHandler as HTTPBasicAuthHandler
from urllib.request import ProxyBasicAuthHandler as ProxyBasicAuthHandler
from urllib.request import AbstractDigestAuthHandler as AbstractDigestAuthHandler
from urllib.request import HTTPDigestAuthHandler as HTTPDigestAuthHandler
from urllib.request import ProxyDigestAuthHandler as ProxyDigestAuthHandler
from urllib.request import HTTPHandler as HTTPHandler
from urllib.request import HTTPSHandler as HTTPSHandler
from urllib.request import FileHandler as FileHandler
from urllib.request import FTPHandler as FTPHandler
from urllib.request import CacheFTPHandler as CacheFTPHandler
from urllib.request import UnknownHandler as UnknownHandler
from urllib.request import HTTPErrorProcessor as HTTPErrorProcessor
from urllib.request import urlretrieve as urlretrieve
from urllib.request import urlcleanup as urlcleanup
from urllib.request import URLopener as URLopener
from urllib.request import FancyURLopener as FancyURLopener
# from urllib.request import proxy_bypass as proxy_bypass



"""
A shared state for all TypeInfos that holds global cache and dependency information,
and potentially other mutable TypeInfo state. This module contains mutable global state.
"""

from typing import Dict, Set, Tuple

MYPY = False
if MYPY:
    from typing import ClassVar
from mypy.nodes import TypeInfo
from mypy.types import Instance


class TypeState:
    """
    This class provides subtype caching to improve performance of subtype checks.

    Note: to avoid leaking global state, 'reset_all_subtype_caches()' should be called
    after a build has finished and after a daemon shutdown. This class only exists for
    performance reasons, resetting subtype caches for a class has no semantic effect.
    """

    # 'caches' and 'caches_proper' are subtype caches, implemented as sets of pairs
    # of (subtype, supertype), where supertypes are instances of given TypeInfo.
    # We need the caches, since subtype checks for structural types are very slow.
    # _subtype_caches_proper is for caching proper subtype checks (i.e. not assuming that
    # Any is consistent with every type).
    _subtype_caches = {}  # type: ClassVar[Dict[TypeInfo, Set[Tuple[Instance, Instance]]]]
    _subtype_caches_proper = {}  # type: ClassVar[Dict[TypeInfo, Set[Tuple[Instance, Instance]]]]

    @classmethod
    def reset_all_subtype_caches(cls) -> None:
        """Completely reset all known subtype caches."""
        cls._subtype_caches = {}
        cls._subtype_caches_proper = {}

    @classmethod
    def reset_subtype_caches_for(cls, info: TypeInfo) -> None:
        """Reset subtype caches (if any) for a given supertype TypeInfo."""
        cls._subtype_caches.setdefault(info, set()).clear()
        cls._subtype_caches_proper.setdefault(info, set()).clear()

    @classmethod
    def reset_all_subtype_caches_for(cls, info: TypeInfo) -> None:
        """Reset subtype caches (if any) for a given supertype TypeInfo and its MRO."""
        for item in info.mro:
            cls.reset_subtype_caches_for(item)

    @classmethod
    def is_cached_subtype_check(cls, left: Instance, right: Instance) -> bool:
        return (left, right) in cls._subtype_caches.setdefault(right.type, set())

    @classmethod
    def is_cached_proper_subtype_check(cls, left: Instance, right: Instance) -> bool:
        return (left, right) in cls._subtype_caches_proper.setdefault(right.type, set())

    @classmethod
    def record_subtype_cache_entry(cls, left: Instance, right: Instance) -> None:
        cls._subtype_caches.setdefault(right.type, set()).add((left, right))

    @classmethod
    def record_proper_subtype_cache_entry(cls, left: Instance, right: Instance) -> None:
        cls._subtype_caches_proper.setdefault(right.type, set()).add((left, right))



from typing import Any

Cc = ...  # type: str
Cf = ...  # type: str
Cn = ...  # type: str
Co = ...  # type: str
Cs = ...  # type: Any
Ll = ...  # type: str
Lm = ...  # type: str
Lo = ...  # type: str
Lt = ...  # type: str
Lu = ...  # type: str
Mc = ...  # type: str
Me = ...  # type: str
Mn = ...  # type: str
Nd = ...  # type: str
Nl = ...  # type: str
No = ...  # type: str
Pc = ...  # type: str
Pd = ...  # type: str
Pe = ...  # type: str
Pf = ...  # type: str
Pi = ...  # type: str
Po = ...  # type: str
Ps = ...  # type: str
Sc = ...  # type: str
Sk = ...  # type: str
Sm = ...  # type: str
So = ...  # type: str
Zl = ...  # type: str
Zp = ...  # type: str
Zs = ...  # type: str
cats = ...  # type: Any

def combine(*args): ...

xid_start = ...  # type: str
xid_continue = ...  # type: str

def allexcept(*args): ...



from typing import List, Dict, Optional, Sequence

import sys

cmp_op = ...  # type: Sequence[str]
hasconst = ...  # type: List[int]
hasname = ...  # type: List[int]
hasjrel = ...  # type: List[int]
hasjabs = ...  # type: List[int]
haslocal = ...  # type: List[int]
hascompare = ...  # type: List[int]
hasfree = ...  # type: List[int]
opname = ...  # type: List[str]

opmap = ...  # type: Dict[str, int]
HAVE_ARGUMENT = ...  # type: int
EXTENDED_ARG = ...  # type: int

if sys.version_info >= (3, 4):
    def stack_effect(opcode: int, oparg: Optional[int] = ...) -> int: ...

if sys.version_info >= (3, 6):
    hasnargs = ...  # type: List[int]



from typing import List, Any, TypeVar, Union, Iterable, Iterator, TypeVar, Generic, Type, Sized, Reversible, Container, Mapping
from abc import ABCMeta

_T = TypeVar('_T')
_S = TypeVar('_S', bound=Type[Enum])

# Note: EnumMeta actually subclasses type directly, not ABCMeta.
# This is a temporary workaround to allow multiple creation of enums with builtins
# such as str as mixins, which due to the handling of ABCs of builtin types, cause
# spurious inconsistent metaclass structure. See #1595.
class EnumMeta(ABCMeta, Iterable[Enum], Sized, Reversible[Enum], Container[Enum]):
    def __iter__(self: Type[_T]) -> Iterator[_T]: ...
    def __reversed__(self: Type[_T]) -> Iterator[_T]: ...
    def __contains__(self, member: Any) -> bool: ...
    def __getitem__(self: Type[_T], name: str) -> _T: ...
    @property
    def __members__(self: Type[_T]) -> Mapping[str, _T]: ...

class Enum(metaclass=EnumMeta):
    def __new__(cls: Type[_T], value: Any) -> _T: ...
    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    def __dir__(self) -> List[str]: ...
    def __format__(self, format_spec: str) -> str: ...
    def __hash__(self) -> Any: ...
    def __reduce_ex__(self, proto: Any) -> Any: ...

    name = ...  # type: str
    value = ...  # type: Any

class IntEnum(int, Enum): ...

def unique(enumeration: _S) -> _S: ...



from typing import Any
import rfc822

class Message(rfc822.Message):
    encodingheader = ...  # type: Any
    typeheader = ...  # type: Any
    def __init__(self, fp, seekable=1): ...
    plisttext = ...  # type: Any
    type = ...  # type: Any
    maintype = ...  # type: Any
    subtype = ...  # type: Any
    def parsetype(self): ...
    plist = ...  # type: Any
    def parseplist(self): ...
    def getplist(self): ...
    def getparam(self, name): ...
    def getparamnames(self): ...
    def getencoding(self): ...
    def gettype(self): ...
    def getmaintype(self): ...
    def getsubtype(self): ...

def choose_boundary(): ...
def decode(input, output, encoding): ...
def encode(input, output, encoding): ...
def copyliteral(input, output): ...
def copybinary(input, output): ...



"""Semantic analyzer test cases"""

import os.path

from typing import Dict, List

from mypy import build
from mypy.build import BuildSource
from mypy.defaults import PYTHON3_VERSION
from mypy.test.helpers import (
    assert_string_arrays_equal, normalize_error_messages, testfile_pyversion,
)
from mypy.test.data import DataDrivenTestCase, DataSuite
from mypy.test.config import test_temp_dir
from mypy.errors import CompileError
from mypy.nodes import TypeInfo
from mypy.options import Options


# Semantic analyzer test cases: dump parse tree

# Semantic analysis test case description files.
semanal_files = ['semanal-basic.test',
                 'semanal-expressions.test',
                 'semanal-classes.test',
                 'semanal-types.test',
                 'semanal-typealiases.test',
                 'semanal-modules.test',
                 'semanal-statements.test',
                 'semanal-abstractclasses.test',
                 'semanal-namedtuple.test',
                 'semanal-typeddict.test',
                 'semanal-classvar.test',
                 'semanal-python2.test']


def get_semanal_options() -> Options:
    options = Options()
    options.use_builtins_fixtures = True
    options.semantic_analysis_only = True
    options.show_traceback = True
    options.python_version = PYTHON3_VERSION
    return options


class SemAnalSuite(DataSuite):
    files = semanal_files
    base_path = test_temp_dir
    optional_out = True
    native_sep = True

    def run_case(self, testcase: DataDrivenTestCase) -> None:
        test_semanal(testcase)


def test_semanal(testcase: DataDrivenTestCase) -> None:
    """Perform a semantic analysis test case.

    The testcase argument contains a description of the test case
    (inputs and output).
    """

    try:
        src = '\n'.join(testcase.input)
        options = get_semanal_options()
        options.python_version = testfile_pyversion(testcase.file)
        result = build.build(sources=[BuildSource('main', None, src)],
                             options=options,
                             alt_lib_path=test_temp_dir)
        a = result.errors
        if a:
            raise CompileError(a)
        # Include string representations of the source files in the actual
        # output.
        for fnam in sorted(result.files.keys()):
            f = result.files[fnam]
            # Omit the builtins module and files with a special marker in the
            # path.
            # TODO the test is not reliable
            if (not f.path.endswith((os.sep + 'builtins.pyi',
                                     'typing.pyi',
                                     'mypy_extensions.pyi',
                                     'abc.pyi',
                                     'collections.pyi'))
                    and not os.path.basename(f.path).startswith('_')
                    and not os.path.splitext(
                        os.path.basename(f.path))[0].endswith('_')):
                a += str(f).split('\n')
    except CompileError as e:
        a = e.messages
    a = normalize_error_messages(a)
    assert_string_arrays_equal(
        testcase.output, a,
        'Invalid semantic analyzer output ({}, line {})'.format(testcase.file,
                                                                testcase.line))


# Semantic analyzer error test cases

class SemAnalErrorSuite(DataSuite):
    files = ['semanal-errors.test']
    base_path = test_temp_dir
    optional_out = True

    def run_case(self, testcase: DataDrivenTestCase) -> None:
        test_semanal_error(testcase)


def test_semanal_error(testcase: DataDrivenTestCase) -> None:
    """Perform a test case."""

    try:
        src = '\n'.join(testcase.input)
        res = build.build(sources=[BuildSource('main', None, src)],
                          options=get_semanal_options(),
                          alt_lib_path=test_temp_dir)
        a = res.errors
        assert a, 'No errors reported in {}, line {}'.format(testcase.file, testcase.line)
    except CompileError as e:
        # Verify that there was a compile error and that the error messages
        # are equivalent.
        a = e.messages
    assert_string_arrays_equal(
        testcase.output, normalize_error_messages(a),
        'Invalid compiler output ({}, line {})'.format(testcase.file, testcase.line))


# SymbolNode table export test cases

class SemAnalSymtableSuite(DataSuite):
    files = ['semanal-symtable.test']
    base_path = test_temp_dir

    def run_case(self, testcase: DataDrivenTestCase) -> None:
        """Perform a test case."""
        try:
            # Build test case input.
            src = '\n'.join(testcase.input)
            result = build.build(sources=[BuildSource('main', None, src)],
                                 options=get_semanal_options(),
                                 alt_lib_path=test_temp_dir)
            # The output is the symbol table converted into a string.
            a = result.errors
            if a:
                raise CompileError(a)
            for f in sorted(result.files.keys()):
                if f not in ('builtins', 'typing', 'abc'):
                    a.append('{}:'.format(f))
                    for s in str(result.files[f].names).split('\n'):
                        a.append('  ' + s)
        except CompileError as e:
            a = e.messages
        assert_string_arrays_equal(
            testcase.output, a,
            'Invalid semantic analyzer output ({}, line {})'.format(
                testcase.file, testcase.line))


# Type info export test cases
class SemAnalTypeInfoSuite(DataSuite):
    files = ['semanal-typeinfo.test']
    base_path = test_temp_dir

    def run_case(self, testcase: DataDrivenTestCase) -> None:
        """Perform a test case."""
        try:
            # Build test case input.
            src = '\n'.join(testcase.input)
            result = build.build(sources=[BuildSource('main', None, src)],
                                 options=get_semanal_options(),
                                 alt_lib_path=test_temp_dir)
            a = result.errors
            if a:
                raise CompileError(a)

            # Collect all TypeInfos in top-level modules.
            typeinfos = TypeInfoMap()
            for f in result.files.values():
                for n in f.names.values():
                    if isinstance(n.node, TypeInfo):
                        assert n.fullname is not None
                        typeinfos[n.fullname] = n.node

            # The output is the symbol table converted into a string.
            a = str(typeinfos).split('\n')
        except CompileError as e:
            a = e.messages
        assert_string_arrays_equal(
            testcase.output, a,
            'Invalid semantic analyzer output ({}, line {})'.format(
                testcase.file, testcase.line))


class TypeInfoMap(Dict[str, TypeInfo]):
    def __str__(self) -> str:
        a = ['TypeInfoMap(']  # type: List[str]
        for x, y in sorted(self.items()):
            if isinstance(x, str) and (not x.startswith('builtins.') and
                                       not x.startswith('typing.') and
                                       not x.startswith('abc.')):
                ti = ('\n' + '  ').join(str(y).split('\n'))
                a.append('  {} : {}'.format(x, ti))
        a[-1] += ')'
        return '\n'.join(a)



"""Generate fine-grained dependencies for AST nodes, for use in the daemon mode.

Dependencies are stored in a map from *triggers* to *sets of affected locations*.

A trigger is a string that represents a program property that has changed, such
as the signature of a specific function. Triggers are written as '<...>' (angle
brackets). When a program property changes, we determine the relevant trigger(s)
and all affected locations. The latter are stale and will have to be reprocessed.

An affected location is a string than can refer to a *target* (a non-nested
function or method, or a module top level), a class, or a trigger (for
recursively triggering other triggers).

Here's an example represention of a simple dependency map (in format
"<trigger> -> locations"):

  <m.A.g> -> m.f
  <m.A> -> <m.f>, m.A, m.f

Assuming 'A' is a class, this means that

1) if a property of 'm.A.g', such as the signature, is changed, we need
   to process target (function) 'm.f'

2) if the MRO or other significant property of class 'm.A' changes, we
   need to process target 'm.f', the entire class 'm.A', and locations
   triggered by trigger '<m.f>' (this explanation is a bit simplified;
   see below for more details).

The triggers to fire are determined using mypy.server.astdiff.

Examples of triggers:

* '<mod.x>' represents a module attribute/function/class. If any externally
  visible property of 'x' changes, this gets fired. For changes within
  classes, only "big" changes cause the class to be triggered (such as a
  change in MRO). Smaller changes, such as changes to some attributes, don't
  trigger the entire class.
* '<mod.Cls.x>' represents the type and kind of attribute/method 'x' of
  class 'mod.Cls'. This can also refer to an attribute inherited from a
  base class (relevant if it's accessed through a value of type 'Cls'
  instead of the base class type).
* '<package.mod>' represents the existence of module 'package.mod'. This
  gets triggered if 'package.mod' is created or deleted, or if it gets
  changed into something other than a module.

Examples of locations:

* 'mod' is the top level of module 'mod' (doesn't include any function bodies,
  but includes class bodies not nested within a function).
* 'mod.f' is function 'f' in module 'mod' (module-level variables aren't separate
  locations but are included in the module top level). Functions also include
  any nested functions and classes -- such nested definitions aren't separate
  locations, for simplicity of implementation.
* 'mod.Cls.f' is method 'f' of 'mod.Cls'. Non-method attributes aren't locations.
* 'mod.Cls' represents each method in class 'mod.Cls' + the top-level of the
  module 'mod'. (To simplify the implementation, there is no location that only
  includes the body of a class without the entire surrounding module top level.)
* Trigger '<...>' as a location is an indirect way of referring to to all
  locations triggered by the trigger. These indirect locations keep the
  dependency map smaller and easier to manage.

Triggers can be triggered by program changes such as these:

* Addition or deletion of an attribute (or module).
* Change of the kind of thing a name represents (such as a change from a function
  to a class).
* Change of the static type of a name.

Changes in the body of a function that aren't reflected in the signature don't
cause the function to be triggered. More generally, we trigger only on changes
that may affect type checking results outside the module that contains the
change.

We don't generate dependencies from builtins and certain other stdlib modules,
since these change very rarely, and they would just increase the size of the
dependency map significantly without significant benefit.

Test cases for this module live in 'test-data/unit/deps*.test'.
"""

from typing import Dict, List, Set, Optional, Tuple, Union

MYPY = False
if MYPY:
    from typing import DefaultDict

from mypy.checkmember import bind_self
from mypy.nodes import (
    Node, Expression, MypyFile, FuncDef, ClassDef, AssignmentStmt, NameExpr, MemberExpr, Import,
    ImportFrom, CallExpr, CastExpr, TypeVarExpr, TypeApplication, IndexExpr, UnaryExpr, OpExpr,
    ComparisonExpr, GeneratorExpr, DictionaryComprehension, StarExpr, PrintStmt, ForStmt, WithStmt,
    TupleExpr, ListExpr, OperatorAssignmentStmt, DelStmt, YieldFromExpr, Decorator, Block,
    TypeInfo, FuncBase, OverloadedFuncDef, RefExpr, SuperExpr, Var, NamedTupleExpr, TypedDictExpr,
    LDEF, MDEF, GDEF, FuncItem, TypeAliasExpr, NewTypeExpr, ImportAll, EnumCallExpr, AwaitExpr,
    op_methods, reverse_op_methods, ops_with_inplace_method, unary_op_methods
)
from mypy.traverser import TraverserVisitor
from mypy.types import (
    Type, Instance, AnyType, NoneTyp, TypeVisitor, CallableType, DeletedType, PartialType,
    TupleType, TypeType, TypeVarType, TypedDictType, UnboundType, UninhabitedType, UnionType,
    FunctionLike, ForwardRef, Overloaded
)
from mypy.server.trigger import make_trigger, make_wildcard_trigger
from mypy.util import correct_relative_import
from mypy.scope import Scope


def get_dependencies(target: MypyFile,
                     type_map: Dict[Expression, Type],
                     python_version: Tuple[int, int]) -> Dict[str, Set[str]]:
    """Get all dependencies of a node, recursively."""
    visitor = DependencyVisitor(type_map, python_version, target.alias_deps)
    target.accept(visitor)
    return visitor.map


def get_dependencies_of_target(module_id: str,
                               module_tree: MypyFile,
                               target: Node,
                               type_map: Dict[Expression, Type],
                               python_version: Tuple[int, int]) -> Dict[str, Set[str]]:
    """Get dependencies of a target -- don't recursive into nested targets."""
    # TODO: Add tests for this function.
    visitor = DependencyVisitor(type_map, python_version, module_tree.alias_deps)
    visitor.scope.enter_file(module_id)
    if isinstance(target, MypyFile):
        # Only get dependencies of the top-level of the module. Don't recurse into
        # functions.
        for defn in target.defs:
            # TODO: Recurse into top-level statements and class bodies but skip functions.
            if not isinstance(defn, (ClassDef, Decorator, FuncDef, OverloadedFuncDef)):
                defn.accept(visitor)
    elif isinstance(target, FuncBase) and target.info:
        # It's a method.
        # TODO: Methods in nested classes.
        visitor.scope.enter_class(target.info)
        target.accept(visitor)
        visitor.scope.leave()
    else:
        target.accept(visitor)
    visitor.scope.leave()
    return visitor.map


class DependencyVisitor(TraverserVisitor):
    def __init__(self,
                 type_map: Dict[Expression, Type],
                 python_version: Tuple[int, int],
                 alias_deps: 'DefaultDict[str, Set[str]]') -> None:
        self.scope = Scope()
        self.type_map = type_map
        self.python2 = python_version[0] == 2
        # This attribute holds a mapping from target to names of type aliases
        # it depends on. These need to be processed specially, since they are
        # only present in expanded form in symbol tables. For example, after:
        #    A = List[int]
        #    x: A
        # The module symbol table will just have a Var `x` with type `List[int]`,
        # and the dependency of `x` on `A` is lost. Therefore the alias dependencies
        # are preserved at alias expansion points in `semanal.py`, stored as an attribute
        # on MypyFile, and then passed here.
        self.alias_deps = alias_deps
        self.map = {}  # type: Dict[str, Set[str]]
        self.is_class = False
        self.is_package_init_file = False

    # TODO (incomplete):
    #   protocols

    def visit_mypy_file(self, o: MypyFile) -> None:
        self.scope.enter_file(o.fullname())
        self.is_package_init_file = o.is_package_init_file()
        self.add_type_alias_deps(self.scope.current_target())
        super().visit_mypy_file(o)
        self.scope.leave()

    def visit_func_def(self, o: FuncDef) -> None:
        self.scope.enter_function(o)
        target = self.scope.current_target()
        if o.type:
            if self.is_class and isinstance(o.type, FunctionLike):
                signature = bind_self(o.type)  # type: Type
            else:
                signature = o.type
            for trigger in get_type_triggers(signature):
                self.add_dependency(trigger)
                self.add_dependency(trigger, target=make_trigger(target))
        if o.info:
            for base in non_trivial_bases(o.info):
                self.add_dependency(make_trigger(base.fullname() + '.' + o.name()))
        self.add_type_alias_deps(self.scope.current_target())
        super().visit_func_def(o)
        variants = set(o.expanded) - {o}
        for ex in variants:
            if isinstance(ex, FuncDef):
                super().visit_func_def(ex)
        self.scope.leave()

    def visit_decorator(self, o: Decorator) -> None:
        # We don't need to recheck outer scope for an overload, only overload itself.
        # Also if any decorator is nested, it is not externally visible, so we don't need to
        # generate dependency.
        if not o.func.is_overload and self.scope.current_function_name() is None:
            self.add_dependency(make_trigger(o.func.fullname()))
        super().visit_decorator(o)

    def visit_class_def(self, o: ClassDef) -> None:
        self.scope.enter_class(o.info)
        target = self.scope.current_full_target()
        self.add_dependency(make_trigger(target), target)
        old_is_class = self.is_class
        self.is_class = True
        # Add dependencies to type variables of a generic class.
        for tv in o.type_vars:
            self.add_dependency(make_trigger(tv.fullname), target)
        self.process_type_info(o.info)
        super().visit_class_def(o)
        self.is_class = old_is_class
        self.scope.leave()

    def visit_newtype_expr(self, o: NewTypeExpr) -> None:
        if o.info:
            self.scope.enter_class(o.info)
            self.process_type_info(o.info)
            self.scope.leave()

    def process_type_info(self, info: TypeInfo) -> None:
        target = self.scope.current_full_target()
        for base in info.bases:
            self.add_type_dependencies(base, target=target)
        if info.tuple_type:
            self.add_type_dependencies(info.tuple_type, target=make_trigger(target))
        if info.typeddict_type:
            self.add_type_dependencies(info.typeddict_type, target=make_trigger(target))
        if info.declared_metaclass:
            self.add_type_dependencies(info.declared_metaclass, target=make_trigger(target))
        self.add_type_alias_deps(self.scope.current_target())
        for name, node in info.names.items():
            if isinstance(node.node, Var):
                # Recheck Liskov if needed, self definitions are checked in the defining method
                if node.node.is_initialized_in_class and has_user_bases(info):
                    self.add_dependency(make_trigger(info.fullname() + '.' + name))
                for base_info in non_trivial_bases(info):
                    # If the type of an attribute changes in a base class, we make references
                    # to the attribute in the subclass stale.
                    self.add_dependency(make_trigger(base_info.fullname() + '.' + name),
                                        target=make_trigger(info.fullname() + '.' + name))
        for base_info in non_trivial_bases(info):
            for name, node in base_info.names.items():
                self.add_dependency(make_trigger(base_info.fullname() + '.' + name),
                                    target=make_trigger(info.fullname() + '.' + name))
            self.add_dependency(make_trigger(base_info.fullname() + '.__init__'),
                                target=make_trigger(info.fullname() + '.__init__'))
            self.add_dependency(make_trigger(base_info.fullname() + '.__new__'),
                                target=make_trigger(info.fullname() + '.__new__'))
            # If the set of abstract attributes change, this may invalidate class
            # instantiation, or change the generated error message, since Python checks
            # class abstract status when creating an instance.
            #
            # TODO: We should probably add this dependency only from the __init__ of the
            #     current class, and independent of bases (to trigger changes in message
            #     wording, as errors may enumerate all abstract attributes).
            self.add_dependency(make_trigger(base_info.fullname() + '.(abstract)'),
                                target=make_trigger(info.fullname() + '.__init__'))
            # If the base class abstract attributes change, subclass abstract
            # attributes need to be recalculated.
            self.add_dependency(make_trigger(base_info.fullname() + '.(abstract)'))

    def visit_import(self, o: Import) -> None:
        for id, as_id in o.ids:
            self.add_dependency(make_trigger(id), self.scope.current_target())

    def visit_import_from(self, o: ImportFrom) -> None:
        module_id, _ = correct_relative_import(self.scope.current_module_id(),
                                               o.relative,
                                               o.id,
                                               self.is_package_init_file)
        for name, as_name in o.names:
            self.add_dependency(make_trigger(module_id + '.' + name))

    def visit_import_all(self, o: ImportAll) -> None:
        module_id, _ = correct_relative_import(self.scope.current_module_id(),
                                               o.relative,
                                               o.id,
                                               self.is_package_init_file)
        # The current target needs to be rechecked if anything "significant" changes in the
        # target module namespace (as the imported definitions will need to be updated).
        self.add_dependency(make_wildcard_trigger(module_id))

    def visit_block(self, o: Block) -> None:
        if not o.is_unreachable:
            super().visit_block(o)

    def visit_assignment_stmt(self, o: AssignmentStmt) -> None:
        rvalue = o.rvalue
        if isinstance(rvalue, CallExpr) and isinstance(rvalue.analyzed, TypeVarExpr):
            analyzed = rvalue.analyzed
            self.add_type_dependencies(analyzed.upper_bound,
                                       target=make_trigger(analyzed.fullname()))
            for val in analyzed.values:
                self.add_type_dependencies(val, target=make_trigger(analyzed.fullname()))
            # We need to re-analyze the definition if bound or value is deleted.
            super().visit_call_expr(rvalue)
        elif isinstance(rvalue, CallExpr) and isinstance(rvalue.analyzed, NamedTupleExpr):
            # Depend on types of named tuple items.
            info = rvalue.analyzed.info
            prefix = '%s.%s' % (self.scope.current_full_target(), info.name())
            for name, symnode in info.names.items():
                if not name.startswith('_') and isinstance(symnode.node, Var):
                    typ = symnode.node.type
                    if typ:
                        self.add_type_dependencies(typ)
                        self.add_type_dependencies(typ, target=make_trigger(prefix))
                        attr_target = make_trigger('%s.%s' % (prefix, name))
                        self.add_type_dependencies(typ, target=attr_target)
        elif isinstance(rvalue, CallExpr) and isinstance(rvalue.analyzed, TypedDictExpr):
            # Depend on the underlying typeddict type
            info = rvalue.analyzed.info
            assert info.typeddict_type is not None
            prefix = '%s.%s' % (self.scope.current_full_target(), info.name())
            self.add_type_dependencies(info.typeddict_type, target=make_trigger(prefix))
        elif isinstance(rvalue, CallExpr) and isinstance(rvalue.analyzed, EnumCallExpr):
            # Enum values are currently not checked, but for future we add the deps on them
            for name, symnode in rvalue.analyzed.info.names.items():
                if isinstance(symnode.node, Var) and symnode.node.type:
                    self.add_type_dependencies(symnode.node.type)
        elif o.is_alias_def:
            assert len(o.lvalues) == 1
            lvalue = o.lvalues[0]
            assert isinstance(lvalue, NameExpr)
            # TODO: get rid of this extra dependency from __init__ to alias definition scope
            typ = self.type_map.get(lvalue)
            if isinstance(typ, FunctionLike) and typ.is_type_obj():
                class_name = typ.type_object().fullname()
                self.add_dependency(make_trigger(class_name + '.__init__'))
                self.add_dependency(make_trigger(class_name + '.__new__'))
            if isinstance(rvalue, IndexExpr) and isinstance(rvalue.analyzed, TypeAliasExpr):
                self.add_type_dependencies(rvalue.analyzed.type)
        else:
            # Normal assignment
            super().visit_assignment_stmt(o)
            for lvalue in o.lvalues:
                self.process_lvalue(lvalue)
            items = o.lvalues + [rvalue]
            for i in range(len(items) - 1):
                lvalue = items[i]
                rvalue = items[i + 1]
                if isinstance(lvalue, TupleExpr):
                    self.add_attribute_dependency_for_expr(rvalue, '__iter__')
            if o.type:
                for trigger in get_type_triggers(o.type):
                    self.add_dependency(trigger)

    def process_lvalue(self, lvalue: Expression) -> None:
        """Generate additional dependencies for an lvalue."""
        if isinstance(lvalue, IndexExpr):
            self.add_operator_method_dependency(lvalue.base, '__setitem__')
        elif isinstance(lvalue, NameExpr):
            if lvalue.kind in (MDEF, GDEF):
                # Assignment to an attribute in the class body, or direct assignment to a
                # global variable.
                lvalue_type = self.get_non_partial_lvalue_type(lvalue)
                type_triggers = get_type_triggers(lvalue_type)
                attr_trigger = make_trigger('%s.%s' % (self.scope.current_full_target(),
                                                       lvalue.name))
                for type_trigger in type_triggers:
                    self.add_dependency(type_trigger, attr_trigger)
        elif isinstance(lvalue, MemberExpr):
            if self.is_self_member_ref(lvalue) and lvalue.is_new_def:
                node = lvalue.node
                if isinstance(node, Var):
                    info = node.info
                    if info and has_user_bases(info):
                        # Recheck Liskov for self definitions
                        self.add_dependency(make_trigger(info.fullname() + '.' + lvalue.name))
            if lvalue.kind is None:
                # Reference to a non-module attribute
                if lvalue.expr not in self.type_map:
                    # Unreachable assignment -> not checked so no dependencies to generate.
                    return
                object_type = self.type_map[lvalue.expr]
                lvalue_type = self.get_non_partial_lvalue_type(lvalue)
                type_triggers = get_type_triggers(lvalue_type)
                for attr_trigger in self.attribute_triggers(object_type, lvalue.name):
                    for type_trigger in type_triggers:
                        self.add_dependency(type_trigger, attr_trigger)
        elif isinstance(lvalue, TupleExpr):
            for item in lvalue.items:
                self.process_lvalue(item)
        elif isinstance(lvalue, StarExpr):
            self.process_lvalue(lvalue.expr)

    def is_self_member_ref(self, memberexpr: MemberExpr) -> bool:
        """Does memberexpr to refer to an attribute of self?"""
        if not isinstance(memberexpr.expr, NameExpr):
            return False
        node = memberexpr.expr.node
        return isinstance(node, Var) and node.is_self

    def get_non_partial_lvalue_type(self, lvalue: RefExpr) -> Type:
        if lvalue not in self.type_map:
            # Likely a block considered unreachable during type checking.
            return UninhabitedType()
        lvalue_type = self.type_map[lvalue]
        if isinstance(lvalue_type, PartialType):
            if isinstance(lvalue.node, Var) and lvalue.node.type:
                lvalue_type = lvalue.node.type
            else:
                # Probably a secondary, non-definition assignment that doesn't
                # result in a non-partial type. We won't be able to infer any
                # dependencies from this so just return something. (The first,
                # definition assignment with a partial type is handled
                # differently, in the semantic analyzer.)
                assert not lvalue.is_new_def
                return UninhabitedType()
        return lvalue_type

    def visit_operator_assignment_stmt(self, o: OperatorAssignmentStmt) -> None:
        super().visit_operator_assignment_stmt(o)
        self.process_lvalue(o.lvalue)
        method = op_methods[o.op]
        self.add_attribute_dependency_for_expr(o.lvalue, method)
        if o.op in ops_with_inplace_method:
            inplace_method = '__i' + method[2:]
            self.add_attribute_dependency_for_expr(o.lvalue, inplace_method)

    def visit_for_stmt(self, o: ForStmt) -> None:
        super().visit_for_stmt(o)
        if not o.is_async:
            # __getitem__ is only used if __iter__ is missing but for simplicity we
            # just always depend on both.
            self.add_attribute_dependency_for_expr(o.expr, '__iter__')
            self.add_attribute_dependency_for_expr(o.expr, '__getitem__')
            if o.inferred_iterator_type:
                if self.python2:
                    method = 'next'
                else:
                    method = '__next__'
                self.add_attribute_dependency(o.inferred_iterator_type, method)
        else:
            self.add_attribute_dependency_for_expr(o.expr, '__aiter__')
            if o.inferred_iterator_type:
                self.add_attribute_dependency(o.inferred_iterator_type, '__anext__')

        self.process_lvalue(o.index)
        if isinstance(o.index, TupleExpr):
            # Process multiple assignment to index variables.
            item_type = o.inferred_item_type
            if item_type:
                # This is similar to above.
                self.add_attribute_dependency(item_type, '__iter__')
                self.add_attribute_dependency(item_type, '__getitem__')
        if o.index_type:
            self.add_type_dependencies(o.index_type)

    def visit_with_stmt(self, o: WithStmt) -> None:
        super().visit_with_stmt(o)
        for e in o.expr:
            if not o.is_async:
                self.add_attribute_dependency_for_expr(e, '__enter__')
                self.add_attribute_dependency_for_expr(e, '__exit__')
            else:
                self.add_attribute_dependency_for_expr(e, '__aenter__')
                self.add_attribute_dependency_for_expr(e, '__aexit__')
        if o.target_type:
            self.add_type_dependencies(o.target_type)

    def visit_print_stmt(self, o: PrintStmt) -> None:
        super().visit_print_stmt(o)
        if o.target:
            self.add_attribute_dependency_for_expr(o.target, 'write')

    def visit_del_stmt(self, o: DelStmt) -> None:
        super().visit_del_stmt(o)
        if isinstance(o.expr, IndexExpr):
            self.add_attribute_dependency_for_expr(o.expr.base, '__delitem__')

    # Expressions

    def process_global_ref_expr(self, o: RefExpr) -> None:
        if o.fullname is not None:
            self.add_dependency(make_trigger(o.fullname))

        # If this is a reference to a type, generate a dependency to its
        # constructor.
        # TODO: avoid generating spurious dependencies for isinstancce checks,
        # except statements, class attribute reference, etc, if perf problem.
        typ = self.type_map.get(o)
        if isinstance(typ, FunctionLike) and typ.is_type_obj():
            class_name = typ.type_object().fullname()
            self.add_dependency(make_trigger(class_name + '.__init__'))
            self.add_dependency(make_trigger(class_name + '.__new__'))

    def visit_name_expr(self, o: NameExpr) -> None:
        if o.kind == LDEF:
            # We don't track depdendencies to local variables, since they
            # aren't externally visible.
            return
        if o.kind == MDEF:
            # Direct reference to member is only possible in the scope that
            # defined the name, so no dependency is required.
            return
        self.process_global_ref_expr(o)

    def visit_member_expr(self, e: MemberExpr) -> None:
        super().visit_member_expr(e)
        if e.kind is not None:
            # Reference to a module attribute
            self.process_global_ref_expr(e)
        else:
            # Reference to a non-module attribute
            if e.expr not in self.type_map:
                # No type available -- this happens for unreachable code. Since it's unreachable,
                # it wasn't type checked and we don't need to generate dependencies.
                return
            typ = self.type_map[e.expr]
            self.add_attribute_dependency(typ, e.name)

    def visit_super_expr(self, e: SuperExpr) -> None:
        super().visit_super_expr(e)
        if e.info is not None:
            self.add_dependency(make_trigger(e.info.fullname() + '.' + e.name))

    def visit_call_expr(self, e: CallExpr) -> None:
        super().visit_call_expr(e)

    def visit_cast_expr(self, e: CastExpr) -> None:
        super().visit_cast_expr(e)
        self.add_type_dependencies(e.type)

    def visit_type_application(self, e: TypeApplication) -> None:
        super().visit_type_application(e)
        for typ in e.types:
            self.add_type_dependencies(typ)

    def visit_index_expr(self, e: IndexExpr) -> None:
        super().visit_index_expr(e)
        self.add_operator_method_dependency(e.base, '__getitem__')

    def visit_unary_expr(self, e: UnaryExpr) -> None:
        super().visit_unary_expr(e)
        if e.op not in unary_op_methods:
            return
        method = unary_op_methods[e.op]
        self.add_operator_method_dependency(e.expr, method)

    def visit_op_expr(self, e: OpExpr) -> None:
        super().visit_op_expr(e)
        self.process_binary_op(e.op, e.left, e.right)

    def visit_comparison_expr(self, e: ComparisonExpr) -> None:
        super().visit_comparison_expr(e)
        for i, op in enumerate(e.operators):
            left = e.operands[i]
            right = e.operands[i + 1]
            self.process_binary_op(op, left, right)
            if self.python2 and op in ('==', '!=', '<', '<=', '>', '>='):
                self.add_operator_method_dependency(left, '__cmp__')
                self.add_operator_method_dependency(right, '__cmp__')

    def process_binary_op(self, op: str, left: Expression, right: Expression) -> None:
        method = op_methods.get(op)
        if method:
            if op == 'in':
                self.add_operator_method_dependency(right, method)
            else:
                self.add_operator_method_dependency(left, method)
                rev_method = reverse_op_methods.get(method)
                if rev_method:
                    self.add_operator_method_dependency(right, rev_method)

    def add_operator_method_dependency(self, e: Expression, method: str) -> None:
        typ = self.type_map.get(e)
        if typ is not None:
            self.add_operator_method_dependency_for_type(typ, method)

    def add_operator_method_dependency_for_type(self, typ: Type, method: str) -> None:
        # Note that operator methods can't be (non-metaclass) methods of type objects
        # (that is, TypeType objects or Callables representing a type).
        if isinstance(typ, TypeVarType):
            typ = typ.upper_bound
        if isinstance(typ, TupleType):
            typ = typ.fallback
        if isinstance(typ, Instance):
            trigger = make_trigger(typ.type.fullname() + '.' + method)
            self.add_dependency(trigger)
        elif isinstance(typ, UnionType):
            for item in typ.items:
                self.add_operator_method_dependency_for_type(item, method)
        elif isinstance(typ, FunctionLike) and typ.is_type_obj():
            self.add_operator_method_dependency_for_type(typ.fallback, method)
        elif isinstance(typ, TypeType):
            if isinstance(typ.item, Instance) and typ.item.type.metaclass_type is not None:
                self.add_operator_method_dependency_for_type(typ.item.type.metaclass_type, method)

    def visit_generator_expr(self, e: GeneratorExpr) -> None:
        super().visit_generator_expr(e)
        for seq in e.sequences:
            self.add_iter_dependency(seq)

    def visit_dictionary_comprehension(self, e: DictionaryComprehension) -> None:
        super().visit_dictionary_comprehension(e)
        for seq in e.sequences:
            self.add_iter_dependency(seq)

    def visit_star_expr(self, e: StarExpr) -> None:
        super().visit_star_expr(e)
        self.add_iter_dependency(e.expr)

    def visit_yield_from_expr(self, e: YieldFromExpr) -> None:
        super().visit_yield_from_expr(e)
        self.add_iter_dependency(e.expr)

    def visit_await_expr(self, e: AwaitExpr) -> None:
        super().visit_await_expr(e)
        self.add_attribute_dependency_for_expr(e.expr, '__await__')

    # Helpers

    def add_type_alias_deps(self, target: str) -> None:
        # Type aliases are special, because some of the dependencies are calculated
        # in semanal.py, before they are expanded.
        if target in self.alias_deps:
            for alias in self.alias_deps[target]:
                self.add_dependency(make_trigger(alias))

    def add_dependency(self, trigger: str, target: Optional[str] = None) -> None:
        """Add dependency from trigger to a target.

        If the target is not given explicitly, use the current target.
        """
        if trigger.startswith(('<builtins.', '<typing.')):
            # Don't track dependencies to certain builtins to keep the size of
            # the dependencies manageable. These dependencies should only
            # change on mypy version updates, which will require a full rebuild
            # anyway.
            return
        if target is None:
            target = self.scope.current_target()
        self.map.setdefault(trigger, set()).add(target)

    def add_type_dependencies(self, typ: Type, target: Optional[str] = None) -> None:
        """Add dependencies to all components of a type.

        Args:
            target: If not None, override the default (current) target of the
                generated dependency.
        """
        # TODO: Use this method in more places where get_type_triggers() + add_dependency()
        #       are called together.
        for trigger in get_type_triggers(typ):
            self.add_dependency(trigger, target)

    def add_attribute_dependency(self, typ: Type, name: str) -> None:
        """Add dependencies for accessing a named attribute of a type."""
        targets = self.attribute_triggers(typ, name)
        for target in targets:
            self.add_dependency(target)

    def attribute_triggers(self, typ: Type, name: str) -> List[str]:
        """Return all triggers associated with the attribute of a type."""
        if isinstance(typ, TypeVarType):
            typ = typ.upper_bound
        if isinstance(typ, TupleType):
            typ = typ.fallback
        if isinstance(typ, Instance):
            member = '%s.%s' % (typ.type.fullname(), name)
            return [make_trigger(member)]
        elif isinstance(typ, FunctionLike) and typ.is_type_obj():
            member = '%s.%s' % (typ.type_object().fullname(), name)
            triggers = [make_trigger(member)]
            triggers.extend(self.attribute_triggers(typ.fallback, name))
            return triggers
        elif isinstance(typ, UnionType):
            targets = []
            for item in typ.items:
                targets.extend(self.attribute_triggers(item, name))
            return targets
        elif isinstance(typ, TypeType):
            triggers = self.attribute_triggers(typ.item, name)
            if isinstance(typ.item, Instance) and typ.item.type.metaclass_type is not None:
                triggers.append(make_trigger('%s.%s' %
                                             (typ.item.type.metaclass_type.type.fullname(),
                                              name)))
            return triggers
        else:
            return []

    def add_attribute_dependency_for_expr(self, e: Expression, name: str) -> None:
        typ = self.type_map.get(e)
        if typ is not None:
            self.add_attribute_dependency(typ, name)

    def add_iter_dependency(self, node: Expression) -> None:
        typ = self.type_map.get(node)
        if typ:
            self.add_attribute_dependency(typ, '__iter__')


def get_type_triggers(typ: Type) -> List[str]:
    """Return all triggers that correspond to a type becoming stale."""
    return typ.accept(TypeTriggersVisitor())


class TypeTriggersVisitor(TypeVisitor[List[str]]):
    def __init__(self) -> None:
        self.deps = []  # type: List[str]

    def visit_instance(self, typ: Instance) -> List[str]:
        trigger = make_trigger(typ.type.fullname())
        triggers = [trigger]
        for arg in typ.args:
            triggers.extend(get_type_triggers(arg))
        return triggers

    def visit_any(self, typ: AnyType) -> List[str]:
        if typ.missing_import_name is not None:
            return [make_trigger(typ.missing_import_name)]
        return []

    def visit_none_type(self, typ: NoneTyp) -> List[str]:
        return []

    def visit_callable_type(self, typ: CallableType) -> List[str]:
        triggers = []
        for arg in typ.arg_types:
            triggers.extend(get_type_triggers(arg))
        triggers.extend(get_type_triggers(typ.ret_type))
        # fallback is a metaclass type for class objects, and is
        # processed separately.
        return triggers

    def visit_overloaded(self, typ: Overloaded) -> List[str]:
        triggers = []
        for item in typ.items():
            triggers.extend(get_type_triggers(item))
        return triggers

    def visit_deleted_type(self, typ: DeletedType) -> List[str]:
        return []

    def visit_partial_type(self, typ: PartialType) -> List[str]:
        assert False, "Should not see a partial type here"

    def visit_tuple_type(self, typ: TupleType) -> List[str]:
        triggers = []
        for item in typ.items:
            triggers.extend(get_type_triggers(item))
        triggers.extend(get_type_triggers(typ.fallback))
        return triggers

    def visit_type_type(self, typ: TypeType) -> List[str]:
        return get_type_triggers(typ.item)

    def visit_forwardref_type(self, typ: ForwardRef) -> List[str]:
        assert False, 'Internal error: Leaked forward reference object {}'.format(typ)

    def visit_type_var(self, typ: TypeVarType) -> List[str]:
        triggers = []
        if typ.fullname:
            triggers.append(make_trigger(typ.fullname))
        if typ.upper_bound:
            triggers.extend(get_type_triggers(typ.upper_bound))
        for val in typ.values:
            triggers.extend(get_type_triggers(val))
        return triggers

    def visit_typeddict_type(self, typ: TypedDictType) -> List[str]:
        triggers = []
        for item in typ.items.values():
            triggers.extend(get_type_triggers(item))
        triggers.extend(get_type_triggers(typ.fallback))
        return triggers

    def visit_unbound_type(self, typ: UnboundType) -> List[str]:
        return []

    def visit_uninhabited_type(self, typ: UninhabitedType) -> List[str]:
        return []

    def visit_union_type(self, typ: UnionType) -> List[str]:
        triggers = []
        for item in typ.items:
            triggers.extend(get_type_triggers(item))
        return triggers


def non_trivial_bases(info: TypeInfo) -> List[TypeInfo]:
    return [base for base in info.mro[1:]
            if base.fullname() != 'builtins.object']


def has_user_bases(info: TypeInfo) -> bool:
    # TODO: skip everything from typeshed?
    return any(base.module_name not in ('builtins', 'typing', 'enum') for base in info.mro[1:])


def dump_all_dependencies(modules: Dict[str, MypyFile],
                          type_map: Dict[Expression, Type],
                          python_version: Tuple[int, int]) -> None:
    """Generate dependencies for all interesting modules and print them to stdout."""
    all_deps = {}  # type: Dict[str, Set[str]]
    for id, node in modules.items():
        # Uncomment for debugging:
        # print('processing', id)
        if id in ('builtins', 'typing') or '/typeshed/' in node.path:
            continue
        assert id == node.fullname()
        deps = get_dependencies(node, type_map, python_version)
        for trigger, targets in deps.items():
            all_deps.setdefault(trigger, set()).update(targets)

    for trigger, targets in sorted(all_deps.items(), key=lambda x: x[0]):
        print(trigger)
        for target in sorted(targets):
            print('    %s' % target)



from typing import Any
from yaml.error import YAMLError

class ResolverError(YAMLError): ...

class BaseResolver:
    DEFAULT_SCALAR_TAG = ...  # type: Any
    DEFAULT_SEQUENCE_TAG = ...  # type: Any
    DEFAULT_MAPPING_TAG = ...  # type: Any
    yaml_implicit_resolvers = ...  # type: Any
    yaml_path_resolvers = ...  # type: Any
    resolver_exact_paths = ...  # type: Any
    resolver_prefix_paths = ...  # type: Any
    def __init__(self) -> None: ...
    @classmethod
    def add_implicit_resolver(cls, tag, regexp, first): ...
    @classmethod
    def add_path_resolver(cls, tag, path, kind=...): ...
    def descend_resolver(self, current_node, current_index): ...
    def ascend_resolver(self): ...
    def check_resolver_prefix(self, depth, path, kind, current_node, current_index): ...
    def resolve(self, kind, value, implicit): ...

class Resolver(BaseResolver): ...



"""Test cases for running mypy programs using a Python interpreter.

Each test case type checks a program then runs it using Python. The
output (stdout) of the program is compared to expected output. Type checking
uses full builtins and other stubs.

Note: Currently Python interpreter paths are hard coded.

Note: These test cases are *not* included in the main test suite, as including
      this suite would slow down the main suite too much.
"""

import os
import os.path
import re
import sys

import pytest  # type: ignore  # no pytest in typeshed

from typing import List

from mypy.defaults import PYTHON3_VERSION
from mypy.test.config import test_temp_dir
from mypy.test.data import DataDrivenTestCase, DataSuite
from mypy.test.helpers import assert_string_arrays_equal, run_command
from mypy.util import try_find_python2_interpreter
from mypy import api

# Path to Python 3 interpreter
python3_path = sys.executable
program_re = re.compile(r'\b_program.py\b')


class PythonEvaluationSuite(DataSuite):
    files = ['pythoneval.test',
             'python2eval.test',
             'pythoneval-asyncio.test']
    base_path = test_temp_dir
    optional_out = True

    def run_case(self, testcase: DataDrivenTestCase) -> None:
        test_python_evaluation(testcase)


def test_python_evaluation(testcase: DataDrivenTestCase) -> None:
    """Runs Mypy in a subprocess.

    If this passes without errors, executes the script again with a given Python
    version.
    """
    assert testcase.old_cwd is not None, "test was not properly set up"
    # TODO: Enable strict optional for these tests
    mypy_cmdline = ['--show-traceback', '--no-site-packages', '--no-strict-optional']
    py2 = testcase.name.lower().endswith('python2')
    if py2:
        mypy_cmdline.append('--py2')
        interpreter = try_find_python2_interpreter()
        if interpreter is None:
            # Skip, can't find a Python 2 interpreter.
            pytest.skip()
            # placate the type checker
            return
    else:
        interpreter = python3_path
        mypy_cmdline.append('--python-version={}'.format('.'.join(map(str, PYTHON3_VERSION))))

    # Write the program to a file.
    program = '_' + testcase.name + '.py'
    program_path = os.path.join(test_temp_dir, program)
    mypy_cmdline.append(program_path)
    with open(program_path, 'w') as file:
        for s in testcase.input:
            file.write('{}\n'.format(s))
    output = []
    # Type check the program.
    out, err, returncode = api.run(mypy_cmdline)
    # split lines, remove newlines, and remove directory of test case
    for line in (out + err).splitlines():
        if line.startswith(test_temp_dir + os.sep):
            output.append(line[len(test_temp_dir + os.sep):].rstrip("\r\n"))
        else:
            output.append(line.rstrip("\r\n"))
    if returncode == 0:
        # Execute the program.
        returncode, interp_out = run_command([interpreter, program])
        output.extend(interp_out)
    # Remove temp file.
    os.remove(program_path)
    assert_string_arrays_equal(adapt_output(testcase), output,
                               'Invalid output ({}, line {})'.format(
                                   testcase.file, testcase.line))


def adapt_output(testcase: DataDrivenTestCase) -> List[str]:
    """Translates the generic _program.py into the actual filename."""
    program = '_' + testcase.name + '.py'
    return [program_re.sub(program, line) for line in testcase.output]



"""Stub file for the '_tracemalloc' module."""
# This is an autogenerated file. It serves as a starting point
# for a more precise manual annotation of this module.
# Feel free to edit the source below, but remove this header when you do.

from typing import Any, List, Tuple, Dict, Generic

def _get_object_traceback(*args, **kwargs) -> Any: ...

def _get_traces() -> Any:
    raise MemoryError()

def clear_traces() -> None: ...

def get_traceback_limit() -> int: ...

def get_traced_memory() -> tuple: ...

def get_tracemalloc_memory() -> Any: ...

def is_tracing() -> bool: ...

def start(*args, **kwargs) -> None:
    raise ValueError()

def stop() -> None: ...
